Package: inline
Version: 0.3.21
Date: 2025-01-08
Title: Functions to Inline C, C++, Fortran Function Calls from R
Authors@R: c(person("Oleg", "Sklyar", role = "aut"),
             person("Dirk", "Eddelbuettel", role = c("aut", "cre"), email = "edd@debian.org",
                    comment = c(ORCID = "0000-0001-6419-907X")),
             person("Romain", "Francois", role = "aut",
                    comment = c(ORCID = "0000-0002-2444-4226")),
             person("Mike", "Smith", role = "ctb"),
             person("Duncan", "Murdoch", role = "ctb"),
             person("Karline", "Soetaert", role = "ctb",
                    comment = c(ORCID = "0000-0003-4603-7100")),
             person("Johannes", "Ranke", role = "ctb",
                    comment = c(ORCID = "0000-0003-4371-6538")))
Imports: methods
Suggests: Rcpp, tinytest
Description: Functionality to dynamically define R functions and S4 methods
 with 'inlined' C, C++ or Fortran code supporting the .C and .Call calling
 conventions.
License: LGPL
Copyright: Oleg Sklyar, 2005-2010 and other authors per their commits
LazyLoad: yes
URL: https://github.com/eddelbuettel/inline,
        https://dirk.eddelbuettel.com/code/inline.html
BugReports: https://github.com/eddelbuettel/inline/issues
NeedsCompilation: no
Packaged: 2025-01-08 22:04:33 UTC; edd
Author: Oleg Sklyar [aut],
  Dirk Eddelbuettel [aut, cre] (<https://orcid.org/0000-0001-6419-907X>),
  Romain Francois [aut] (<https://orcid.org/0000-0002-2444-4226>),
  Mike Smith [ctb],
  Duncan Murdoch [ctb],
  Karline Soetaert [ctb] (<https://orcid.org/0000-0003-4603-7100>),
  Johannes Ranke [ctb] (<https://orcid.org/0000-0003-4371-6538>)
Maintainer: Dirk Eddelbuettel <edd@debian.org>
Repository: CRAN
Date/Publication: 2025-01-09 13:00:02 UTC

import("methods")
importFrom("utils", "package.skeleton", "tail")

export(
    "cfunction",
    "cxxfunction",
    "getPlugin",
    "readCFunc",
    "registerPlugin",
    "rcpp",
    "writeCFunc"
)

exportMethods(
    "setCMethod",
    "getDynLib",
    "package.skeleton",
    "moveDLL",
    "print",
    "code"
)

NAME

    Inline - Write Perl Subroutines in Other Programming Languages

VERSION

    This document describes Inline version 0.79. ";

SYNOPSIS

        use Inline C;
    
        print "9 + 16 = ", add(9, 16), "\n";
        print "9 - 16 = ", subtract(9, 16), "\n";
    
        __END__
        __C__
        int add(int x, int y) {
          return x + y;
        }
    
        int subtract(int x, int y) {
          return x - y;
        }

DESCRIPTION

    The Inline module allows you to put source code from other programming
    languages directly "inline" in a Perl script or module. The code is
    automatically compiled as needed, and then loaded for immediate access
    from Perl.

    Inline saves you from the hassle of having to write and compile your
    own glue code using facilities like XS or SWIG. Simply type the code
    where you want it and run your Perl as normal. All the hairy details
    are handled for you. The compilation and installation of your code
    chunks all happen transparently; all you will notice is the delay of
    compilation on the first run.

    The Inline code only gets compiled the first time you run it (or
    whenever it is modified) so you only take the performance hit once.
    Code that is Inlined into distributed modules (like on the CPAN) will
    get compiled when the module is installed, so the end user will never
    notice the compilation time.

    Best of all, it works the same on both Unix and Microsoft Windows. See
    "Inline- Support" for support information.

 Why Inline?

    Do you want to know "Why would I use other languages in Perl?" or "Why
    should I use Inline to do it?"? I'll try to answer both.

    Why would I use other languages in Perl?

      The most obvious reason is performance. For an interpreted language,
      Perl is very fast. Many people will say "Anything Perl can do, C can
      do faster". (They never mention the development time :-) Anyway, you
      may be able to remove a bottleneck in your Perl code by using another
      language, without having to write the entire program in that
      language. This keeps your overall development time down, because
      you're using Perl for all of the non-critical code.

      Another reason is to access functionality from existing API-s that
      use the language. Some of this code may only be available in binary
      form. But by creating small subroutines in the native language, you
      can "glue" existing libraries to your Perl. As a user of the CPAN,
      you know that code reuse is a good thing. So why throw away those
      Fortran libraries just yet?

      If you are using Inline with the C language, then you can access the
      full internals of Perl itself. This opens up the floodgates to both
      extreme power and peril.

      Maybe the best reason is "Because you want to!". Diversity keeps the
      world interesting. TMTOWTDI!

    Why should I use Inline to do it?

      There are already two major facilities for extending Perl with C.
      They are XS and SWIG. Both are similar in their capabilities, at
      least as far as Perl is concerned. And both of them are quite
      difficult to learn compared to Inline.

      There is a big fat learning curve involved with setting up and using
      the XS environment. You need to get quite intimate with the following
      docs:

	* perlxs

	* perlxstut

	* perlapi

	* perlguts

	* perlmod

	* h2xs

	* xsubpp

	* ExtUtils::MakeMaker

      With Inline you can be up and running in minutes. There is a C
      Cookbook with lots of short but complete programs that you can extend
      to your real-life problems. No need to learn about the complicated
      build process going on in the background. You don't even need to
      compile the code yourself. Inline takes care of every last detail
      except writing the C code.

      Perl programmers cannot be bothered with silly things like compiling.
      "Tweak, Run, Tweak, Run" is our way of life. Inline does all the
      dirty work for you.

      Another advantage of Inline is that you can use it directly in a
      script. You can even use it in a Perl one-liner. With XS and SWIG,
      you always set up an entirely separate module. Even if you only have
      one or two functions. Inline makes easy things easy, and hard things
      possible. Just like Perl.

      Finally, Inline supports several programming languages (not just C
      and C++). As of this writing, Inline has support for C, C++, Java,
      Python, Ruby, Tcl, Assembler, Basic, Guile, Befunge, Octave, Awk, BC,
      TT (Template Toolkit), WebChat and even PERL. New Inline Language
      Support Modules (ILSMs) are regularly being added. See Inline-API for
      details on how to create your own ILSM.

USING THE INLINE.PM MODULE

    Inline is a little bit different than most of the Perl modules that you
    are used to. It doesn't import any functions into your namespace and it
    doesn't have any object oriented methods. Its entire interface (with
    two minor exceptions) is specified through the 'use Inline ...'
    command.

    This section will explain all of the different ways to use Inline. If
    you want to begin using C with Inline immediately, see
    Inline::C-Cookbook.

 The Basics

    The most basic form for using Inline is:

        use Inline X => "X source code";

    where 'X' is one of the supported Inline programming languages. The
    second parameter identifies the source code that you want to bind to
    Perl. The source code can be specified using any of the following
    syntaxes:

    The DATA Keyword.

          use Inline Java => 'DATA';
      
          # Perl code goes here ...
      
          __DATA__
          __Java__
          /* Java code goes here ... */

      The easiest and most visually clean way to specify your source code
      in an Inline Perl program is to use the special DATA keyword. This
      tells Inline to look for a special marker in your DATA filehandle's
      input stream. In this example the special marker is __Java__, which
      is the programming language surrounded by double underscores.

      In case you've forgotten, the DATA pseudo file is comprised of all
      the text after the __END__ or __DATA__ section of your program. If
      you're working outside the main package, you'd best use the __DATA__
      marker or else Inline will not find your code.

      Using this scheme keeps your Perl code at the top, and all the ugly
      Java stuff down below where it belongs. This is visually clean and
      makes for more maintainable code. An excellent side benefit is that
      you don't have to escape any characters like you might in a Perl
      string. The source code is verbatim. For these reasons, I prefer this
      method the most.

      The only problem with this style is that since Perl can't read the
      DATA filehandle until runtime, it obviously can't bind your functions
      until runtime. The net effect of this is that you can't use your
      Inline functions as barewords (without predeclaring them) because
      Perl has no idea they exist during compile time.

    The FILE and BELOW keywords.

          use Inline::Files;
          use Inline Java => 'file';
      
          # Perl code goes here ...
      
          __JAVA__
          /* Java code goes here ... */

      This is the newest method of specifying your source code. It makes
      use of the Perl module Inline::Files written by Damian Conway. The
      basic style and meaning are the same as for the DATA keyword, but
      there are a few syntactic and semantic twists.

      First, you must say 'use Inline::Files' before you 'use Inline' code
      that needs those files. The special 'DATA' keyword is replaced by
      either 'file' or 'below'. This allows for the bad pun idiom of:

          use Inline C => 'below';

      You can omit the __DATA__ tag now. Inline::Files is a source filter
      that will remove these sections from your program before Perl
      compiles it. They are then available for Inline to make use of. And
      since this can all be done at compile time, you don't have to worry
      about the caveats of the 'DATA' keyword.

      This module has a couple small gotchas. Since Inline::Files only
      recognizes file markers with capital letters, you must specify the
      capital form of your language name. Also, there is a startup time
      penalty for using a source code filter.

      At this point Inline::Files is alpha software and use of it is
      experimental. Inline's integration of this module is also fledgling
      at the time being. One of things I plan to do with Inline::Files is
      to get line number info so when an extension doesn't compile, the
      error messages will point to the correct source file and line number.

      My best advice is to use Inline::Files for testing (especially as
      support for it improves), but use DATA for production and
      distributed/CPAN code.

    Strings

          use Inline Java => <<'END';
      
          /* Java code goes here ... */
          END
      
          # Perl code goes here ...

      You also just specify the source code as a single string. A handy way
      to write the string is to use Perl's "here document" style of
      quoting. This is ok for small functions but can get unwieldy in the
      large. On the other hand, the string variant probably has the least
      startup penalty and all functions are bound at compile time.

      If you wish to put the string into a scalar variable, please be aware
      that the use statement is a compile time directive. As such, all the
      variables it uses must also be set at compile time, before the 'use
      Inline' statement. Here is one way to do it:

          my $code;
          BEGIN {
              $code = <<END;
      
          /* Java code goes here ... */
          END
          }
          use Inline Java => $code;
      
          # Perl code goes here ...

    The bind() Function

      An alternative to using the BEGIN block method is to specify the
      source code at run time using the 'Inline->bind()' method. (This is
      one of the interface exceptions mentioned above) The bind() method
      takes the same arguments as 'use Inline ...'.

          my $code = <<END;
      
          /* Java code goes here ... */
          END
      
          Inline->bind(Java => $code);

      You can think of bind() as a way to eval() code in other programming
      languages.

      Although bind() is a powerful feature, it is not recommended for use
      in Inline based modules. In fact, it won't work at all for
      installable modules. See instructions below for creating modules with
      Inline.

    Other Methods

      The source code for Inline can also be specified as an external
      filename, a reference to a subroutine that returns source code, or a
      reference to an array that contains lines of source code. (Note that
      if the external source file is in the current directory it must be
      specified with a leading '.' - ie '.file.ext' instead of simply
      'file.ext'.) These methods are less frequently used but may be useful
      in some situations.

    Shorthand

      If you are using the 'DATA' or 'file' methods described above and
      there are no extra parameters, you can omit the keyword altogether.
      For example:

          use Inline 'Java';
      
          # Perl code goes here ...
      
          __DATA__
          __Java__
          /* Java code goes here ... */

      or

          use Inline::Files;
          use Inline 'Java';
      
          # Perl code goes here ...
      
          __JAVA__
          /* Java code goes here ... */

 More about the DATA Section

    If you are writing a module, you can also use the DATA section for POD
    and AutoLoader subroutines. Just be sure to put them before the first
    Inline marker. If you install the helper module Inline::Filters, you
    can even use POD inside your Inline code. You just have to specify a
    filter to strip it out.

    You can also specify multiple Inline sections, possibly in different
    programming languages. Here is another example:

        # The module Foo.pm
        package Foo;
        use AutoLoader;
    
        use Inline C;
        use Inline C => DATA => filters => 'Strip_POD';
        use Inline Python;
    
        1;
    
        __DATA__
    
        sub marine {
            # This is an autoloaded subroutine
        }
    
        =head1 External subroutines
    
        =cut
    
        __C__
        /* First C section */
    
        __C__
        /* Second C section */
        =head1 My C Function
    
        Some POD doc.
    
        =cut
    
        __Python__
        """A Python Section"""

    An important thing to remember is that you need to have one use Inline
    Foo => 'DATA' for each __Foo__ marker, and they must be in the same
    order. This allows you to apply different configuration options to each
    section.

 Configuration Options

    Inline tries to do the right thing as often as possible. But sometimes
    you may need to override the default actions. This is easy to do.
    Simply list the Inline configuration options after the regular Inline
    parameters. All configuration options are specified as (key, value)
    pairs.

        use Inline (C => 'DATA',
                    directory => './inline_dir',
                    libs => '-lfoo',
                    inc => '-I/foo/include',
                    prefix => 'XXX_',
                    warnings => 0,
                   );

    You can also specify the configuration options on a separate Inline
    call like this:

        use Inline (C => Config =>
                    directory => './inline_dir',
                    libs => '-lfoo',
                    inc => '-I/foo/include',
                    prefix => 'XXX_',
                    warnings => 0,
                   );
        use Inline C => <<'END_OF_C_CODE';

    The special keyword 'Config' tells Inline that this is a
    configuration-only call. No source code will be compiled or bound to
    Perl.

    If you want to specify global configuration options that don't apply to
    a particular language, just leave the language out of the call. Like
    this:

        use Inline Config => warnings => 0;

    The Config options are inherited and additive. You can use as many
    Config calls as you want. And you can apply different options to
    different code sections. When a source code section is passed in,
    Inline will apply whichever options have been specified up to that
    point. Here is a complex configuration example:

        use Inline (Config =>
                    directory => './inline_dir',
                   );
        use Inline (C => Config =>
                    libs => '-lglobal',
                   );
        use Inline (C => 'DATA',         # First C Section
                    libs => ['-llocal1', '-llocal2'],
                   );
        use Inline (Config =>
                    warnings => 0,
                   );
        use Inline (Python => 'DATA',    # First Python Section
                    libs => '-lmypython1',
                   );
        use Inline (C => 'DATA',         # Second C Section
                    libs => [undef, '-llocal3'],
                   );

    The first Config applies to all subsequent calls. The second Config
    applies to all subsequent C sections (but not Python sections). In the
    first C section, the external libraries global, local1 and local2 are
    used. (Most options allow either string or array ref forms, and do the
    right thing.) The Python section does not use the global library, but
    does use the same DIRECTORY, and has warnings turned off. The second C
    section only uses the local3 library. That's because a value of undef
    resets the additive behavior.

    The directory and warnings options are generic Inline options. All
    other options are language specific. To find out what the C options do,
    see Inline::C.

 On and Off

    If a particular config option has value options of 1 and 0, you can use
    the 'enable' and 'disable' modifiers. In other words, this:

        use Inline Config =>
                   force_build => 1,
                   clean_after_build => 0;

    could be reworded as:

        use Inline Config =>
                   enable => force_build =>
                   disable => clean_after_build;

 Playing 'with' Others

    Inline has a special configuration syntax that tells it to get more
    configuration options from other Perl modules. Here is an example:

        use Inline with => 'Event';

    This tells Inline to load the module Event.pm and ask it for
    configuration information. Since Event has a C API of its own, it can
    pass Inline all of the information it needs to be able to use Event C
    callbacks seamlessly.

    That means that you don't need to specify the typemaps, shared
    libraries, include files and other information required to get this to
    work.

    You can specify a single module or a list of them. Like:

        use Inline with => qw(Event Foo Bar);

    Currently, Event is the only module that works with Inline.

    In order to make your module work with Inline in this way, your module
    needs to provide a class method called Inline that takes an Inline
    language as a parameter (e.g. "C"), and returns a reference to a hash
    with configuration information that is acceptable to the relevant ILSM.
    For C, see C Configuration Options. E.g.:

        my $confighashref = Event->Inline('C'); # only supports C in 1.21
        # hashref contains keys INC, TYPEMAPS, MYEXTLIB, AUTO_INCLUDE, BOOT

    If your module uses ExtUtils::Depends version 0.400 or higher, your
    module only needs this:

        package Module;
        use autouse Module::Install::Files => qw(Inline);

 Inline Shortcuts

    Inline lets you set many configuration options from the command line.
    These options are called 'shortcuts'. They can be very handy,
    especially when you only want to set the options temporarily, for say,
    debugging.

    For instance, to get some general information about your Inline code in
    the script Foo.pl, use the command:

        perl -MInline=info Foo.pl

    If you want to force your code to compile, even if its already done,
    use:

        perl -MInline=force Foo.pl

    If you want to do both, use:

        perl -MInline=info -MInline=force Foo.pl

    or better yet:

        perl -MInline=info,force Foo.pl

 The Inline 'directory'

    Inline needs a place to build your code and to install the results of
    the build. It uses a single directory named '.Inline/' under normal
    circumstances. If you create this directory in your home directory, the
    current directory or in the directory where your program resides,
    Inline will find and use it. You can also specify it in the environment
    variable PERL_INLINE_DIRECTORY or directly in your program, by using
    the directory keyword option. If Inline cannot find the directory in
    any of these places it will create a '_Inline/' directory in either
    your current directory or the directory where your script resides.

    One of the key factors to using Inline successfully, is understanding
    this directory. When developing code it is usually best to create this
    directory (or let Inline do it) in your current directory. Remember
    that there is nothing sacred about this directory except that it holds
    your compiled code. Feel free to delete it at any time. Inline will
    simply start from scratch and recompile your code on the next run. If
    you have several programs that you want to force to recompile, just
    delete your '.Inline/' directory.

    It is probably best to have a separate '.Inline/' directory for each
    project that you are working on. You may want to keep stable code in
    the <.Inline/> in your home directory. On multi-user systems, each user
    should have their own '.Inline/' directories. It could be a security
    risk to put the directory in a shared place like /tmp/.

 Debugging Inline Errors

    All programmers make mistakes. When you make a mistake with Inline,
    like writing bad C code, you'll get a big error report on your screen.
    This report tells you where to look to do the debugging. Some languages
    may also dump out the error messages generated from the build.

    When Inline needs to build something it creates a subdirectory under
    your DIRECTORY/build/ directory. This is where it writes all the
    components it needs to build your extension. Things like XS files,
    Makefiles and output log files.

    If everything goes OK, Inline will delete this subdirectory. If there
    is an error, Inline will leave the directory intact and print its
    location. The idea is that you are supposed to go into that directory
    and figure out what happened.

    Read the doc for your particular Inline Language Support Module for
    more information.

 The 'config' Registry File

    Inline keeps a cached file of all of the Inline Language Support
    Module's meta data in a file called config. This file can be found in
    your directory directory. If the file does not exist, Inline creates a
    new one. It will search your system for any module beginning with
    Inline::. It will then call that module's register() method to get
    useful information for future invocations.

    Whenever you add a new ILSM, you should delete this file so that Inline
    will auto-discover your newly installed language module. (This should
    no longer be necessary as of Inline-0.49.)

CONFIGURATION OPTIONS

    This section lists all of the generic Inline configuration options. For
    language specific configuration, see the doc for that language.

    directory

      The directory config option is the directory that Inline uses to both
      build and install an extension.

      Normally Inline will search in a bunch of known places for a
      directory called '.Inline/'. Failing that, it will create a directory
      called '_Inline/'

      If you want to specify your own directory, use this configuration
      option.

      Note that you must create the directory directory yourself. Inline
      will not do it for you.

    name

      You can use this option to set the name of your Inline extension
      object module. For example:

          use Inline C => 'DATA',
                     name => 'Foo::Bar';

      would cause your C code to be compiled in to the object:

          lib/auto/Foo/Bar/Bar.so
          lib/auto/Foo/Bar/Bar.inl

      (The .inl component contains dependency information to make sure the
      source code is in sync with the executable)

      If you don't use name, Inline will pick a name for you based on your
      program name or package name. In this case, Inline will also enable
      the autoname option which mangles in a small piece of the MD5
      fingerprint into your object name, to make it unique.

    autoname

      This option is enabled whenever the name parameter is not specified.
      To disable it say:

          use Inline C => 'DATA',
                     disable => 'autoname';

      autoname mangles in enough of the MD5 fingerprint to make your module
      name unique. Objects created with autoname will never get replaced.
      That also means they will never get cleaned up automatically.

      autoname is very useful for small throw away scripts. For more
      serious things, always use the name option.

    version

      Specifies the version number of the Inline extension object. It is
      used only for modules, and it must match the global variable
      $VERSION. Additionally, this option should used if (and only if) a
      module is being set up to be installed permanently into the Perl
      sitelib tree. Inline will croak if you use it otherwise.

      The presence of the version parameter is the official way to let
      Inline know that your code is an installable/installed module. Inline
      will never generate an object in the temporary cache (_Inline/
      directory) if version is set. It will also never try to recompile a
      module that was installed into someone's Perl site tree.

      So the basic rule is develop without version, and deliver with
      version.

    with

      with can also be used as a configuration option instead of using the
      special 'with' syntax. Do this if you want to use different sections
      of Inline code with different modules. (Probably a very rare usage)

          use Event;
          use Inline C => DATA => with => 'Event';

      Modules specified using the config form of with will not be
      automatically required. You must use them yourself.

    using

      You can override modules that get used by ILSMs with the using
      option. This is typically used to override the default parser for
      Inline::C, but might be used by any ILSM for any purpose.

          use Inline config => using => '::Parser::RecDescent';
          use Inline C => '...';

      This would tell Inline::C to use Inline::C::Parser::RecDescent.

    global_load

      This option is for compiled languages only. It tells Inline to tell
      DynaLoader to load an object file in such a way that its symbols can
      be dynamically resolved by other object files. May not work on all
      platforms. See the global shortcut below.

    untaint

      You can use this option whenever you use Perl's -T switch, for taint
      checking. This option tells Inline to blindly untaint all tainted
      variables. (This is generally considered to be an appallingly
      insecure thing to do, and not to be recommended - but the option is
      there for you to use if you want. Please consider using something
      other than Inline for scripts that need taint checking.) It also
      turns on safemode by default. See the untaint shortcut below. You
      will see warnings about blindly untainting fields in both %ENV and
      Inline objects. If you want to silence these warnings, set the Config
      option no_untaint_warn => 1. There can be some problems untainting
      Inline scripts where older versions of Cwd, such as those that
      shipped with early versions of perl-5.8 (and earlier), are installed.
      Updating Cwd will probably solve these problems.

    safemode

      Perform extra safety checking, in an attempt to thwart malicious
      code. This option cannot guarantee security, but it does turn on all
      the currently implemented checks. (Currently, the only "currently
      implemented check" is to ensure that the directory option has also
      been used.)

      There is a slight startup penalty by using safemode. Also, using
      untaint automatically turns this option on. If you need your code to
      start faster under -T (taint) checking, you'll need to turn this
      option off manually. Only do this if you are not worried about
      security risks. See the unsafe shortcut below.

    force_build

      Makes Inline build (compile) the source code every time the program
      is run. The default is 0. See the force shortcut below.

    build_noisy

      Tells ILSMs that they should dump build messages to the terminal
      rather than be silent about all the build details.

    build_timers

      Tells ILSMs to print timing information about how long each build
      phase took. Usually requires Time::HiRes.

    clean_after_build

      Tells Inline to clean up the current build area if the build was
      successful. Sometimes you want to disable this for debugging. Default
      is 1. See the noclean shortcut below.

    clean_build_area

      Tells Inline to clean up the old build areas within the entire Inline
      directory. Default is 0. See the clean shortcut below.

    print_info

      Tells Inline to print various information about the source code.
      Default is 0. See the info shortcut below.

    print_version

      Tells Inline to print version info about itself. Default is 0. See
      the version shortcut below.

    reportbug

      Puts Inline into 'reportbug' mode, which is what you want if you
      desire to report a bug.

    rewrite_config_file

      Default is 0, but setting rewrite_config_file => 1 will mean that the
      existing configuration file in the Inline directory will be
      overwritten. (This is useful if the existing config file is not up to
      date as regards supported languages.)

    warnings

      This option tells Inline whether to print certain warnings. Default
      is 1.

INLINE CONFIGURATION SHORTCUTS

    This is a list of all the shortcut configuration options currently
    available for Inline. Specify them from the command line when running
    Inline scripts.

        perl -MInline=noclean inline_script.pl

    or

        perl -MInline=info,force,noclean inline_script.pl

    You can specify multiple shortcuts separated by commas. They are not
    case sensitive. You can also specify shortcuts inside the Inline
    program like this:

        use Inline 'info', 'force', 'noclean';

    NOTE: If a 'use Inline' statement is used to set shortcuts, it can not
    be used for additional purposes.

    clean

      Tells Inline to remove any build directories that may be lying around
      in your build area. Normally these directories get removed
      immediately after a successful build. Exceptions are when the build
      fails, or when you use the noclean or reportbug options.

    force

      Forces the code to be recompiled, even if everything is up to date.

    global

      Turns on the global_load option.

    info

      This is a very useful option when you want to know what's going on
      under the hood. It tells Inline to print helpful information to
      STDERR. Among the things that get printed is a list of which Inline
      functions were successfully bound to Perl.

    noclean

      Tells Inline to leave the build files after compiling.

    noisy

      Use the build_noisy option to print messages during a build.

    reportbug

      Puts Inline into reportbug mode, which does special processing when
      you want to report a bug. reportbug also automatically forces a
      build, and doesn't clean up afterwards. This is so that you can tar
      and mail the build directory to me. reportbug will print exact
      instructions on what to do. Please read and follow them carefully.

      NOTE: reportbug informs you to use the tar command. If your system
      does not have tar, please use the equivalent zip command.

    safe

      Turns safemode on. untaint will turn this on automatically. While
      this mode performs extra security checking, it does not guarantee
      safety.

    site_install

      This parameter used to be used for creating installable Inline
      modules. It has been removed from Inline altogether and replaced with
      a much simpler and more powerful mechanism, Inline::MakeMaker. See
      the section below on how to create modules with Inline.

    _testing

      Used internally by Ct09parser.t and Ct10callback.t(in the Inline::C
      test suite). Setting this option with Inline::C will mean that files
      named parser_id and void_test are created in the ./Inline_test
      directory, creating that directory if it doesn't already exist. The
      files (but not the ./Inline_test directory) are cleaned up by calling
      Inline::C::_testing_cleanup(). Also used by t/06rewrite_config.t to
      trigger a warning.

    timers

      Turn on build_timers to get extra diagnostic info about builds.

    unsafe

      Turns safemode off. Use this in combination with untaint for slightly
      faster startup time under -T. Only use this if you are sure the
      environment is safe.

    untaint

      Turn the untaint option on. Used with -T switch. In terms of secure
      practices, this is definitely not a recommended way of dealing with
      taint checking, but it's the only option currently available with
      Inline. Use it at your own risk.

    version

      Tells Inline to report its release version.

WRITING MODULES WITH INLINE

    Writing CPAN modules that use C code is easy with Inline. Let's say
    that you wanted to write a module called Math::Simple. Start by using
    the following command:

        h2xs -PAXn Math::Simple

    This will generate a bunch of files that form a skeleton of what you
    need for a distributable module. (Read the h2xs manpage to find out
    what the options do) Next, modify the Simple.pm file to look like this:

        package Math::Simple;
        $VERSION = '1.23';
    
        use base 'Exporter';
        @EXPORT_OK = qw(add subtract);
        use strict;
    
        use Inline C => 'DATA',
                   version => '1.23',
                   name => 'Math::Simple';
    
        # The following Inline->init() call is optional - see below for more info.
        #Inline->init();
    
        1;
    
        __DATA__
    
        =pod
    
        =cut
    
        __C__
        int add(int x, int y) {
          return x + y;
        }
    
        int subtract(int x, int y) {
          return x - y;
        }

    The important things to note here are that you must specify a name and
    version parameter. The name must match your module's package name. The
    version parameter must match your module's $VERSION variable and they
    must be of the form /^\d\.\d\d$/.

    NOTE: These are Inline's sanity checks to make sure you know what
    you're doing before uploading your code to CPAN. They insure that once
    the module has been installed on someone's system, the module would not
    get automatically recompiled for any reason. This makes Inline based
    modules work in exactly the same manner as XS based ones.

    Finally, you need to modify the Makefile.PL. Simply change:

        use ExtUtils::MakeMaker;

    to

        use Inline::MakeMaker;

    And, in order that the module build work correctly in the cpan shell,
    add the following directive to the Makefile.PL's WriteMakefile():

        CONFIGURE_REQUIRES  =>  {
            'Inline::MakeMaker'     => 0.45,
            'ExtUtils::MakeMaker'   => 6.52,
        },

    This CONFIGURE_REQUIRES directive ensures that the cpan shell will
    install Inline on the user's machine (if it's not already present)
    before building your Inline-based module. Specifying of
    "ExtUtils::MakeMaker => 6.52," is optional, and can be omitted if you
    like. It ensures only that some harmless warnings relating to the
    CONFIGURE_REQUIRES directive won't be emitted during the building of
    the module. It also means, of course, that ExtUtils::Makemaker will
    first be updated on the user's machine unless the user already has
    version 6.52 or later.

    If the "Inline->init();" is not done then, having installed
    Math::Simple, a warning that "One or more DATA sections were not
    processed by Inline" will appear when (and only when) Math::Simple is
    loaded by a "require call. It's a harmless warning - and if you're
    prepared to live with it, then there's no need to make the
    "Inline->init();" call.

    When the person installing Math::Simple does a "make", the generated
    Makefile will invoke Inline in such a way that the C code will be
    compiled and the executable code will be placed into the ./blib
    directory. Then when a "make install" is done, the module will be
    copied into the appropriate Perl sitelib directory (which is where an
    installed module should go).

    Now all you need to do is:

        perl Makefile.PL
        make dist

    That will generate the file Math-Simple-0.20.tar.gz which is a
    distributable package. (It will also generate some harmless warnings in
    relation to CONFIGURE_REQUIRES unless the version of your
    ExtUtils::MakeMaker is 6.52 or later.) That's all there is to it.

    IMPORTANT NOTE: Although the above steps will produce a workable
    module, you still have a few more responsibilities as a budding new
    CPAN author. You need to write lots of documentation and write lots of
    tests. Take a look at some of the better CPAN modules for ideas on
    creating a killer test harness. Actually, don't listen to me, go read
    these:

      * perldoc perlnewmod

      * http://www.cpan.org/modules/04pause.html

      * http://www.cpan.org/modules/00modlist.long.html

HOW INLINE WORKS

    In reality, Inline just automates everything you would need to do if
    you were going to do it by hand (using XS, etc).

    Inline performs the following steps:

      * Receive the Source Code

      Inline gets the source code from your script or module with a
      statements like the following:

          use Inline C => "Source-Code";

      or

          use Inline;
          bind Inline C => "Source-Code";

      where C is the programming language of the source code, and Source-
      Code is a string, a file name, an array reference, or the special
      'DATA' keyword.

      Since Inline is coded in a "use" statement, everything is done during
      Perl's compile time. If anything needs to be done that will affect
      the Source- Code, it needs to be done in a BEGIN block that is before
      the "use Inline ..." statement. If you really need to specify code to
      Inline at runtime, you can use the bind() method.

      Source code that is stowed in the 'DATA' section of your code, is
      read in by an INIT subroutine in Inline. That's because the DATA
      filehandle is not available at compile time.

      * Check if the Source Code has been Built

      Inline only needs to build the source code if it has not yet been
      built. It accomplishes this seemingly magical task in an extremely
      simple and straightforward manner. It runs the source text through
      the Digest::MD5 module to produce a 128-bit "fingerprint" which is
      virtually unique. The fingerprint along with a bunch of other
      contingency information is stored in a .inl file that sits next to
      your executable object. For instance, the C code from a script called
      example.pl might create these files:

          example_pl_3a9a.so
          example_pl_3a9a.inl

      If all the contingency information matches the values stored in the
      .inl file, then proceed to step 8. (No compilation is necessary)

      * Find a Place to Build and Install

      At this point Inline knows it needs to build the source code. The
      first thing to figure out is where to create the great big mess
      associated with compilation, and where to put the object when it's
      done.

      By default Inline will try to build and install under the first place
      that meets one of the following conditions:

	1. The DIRECTORY= config option; if specified

	2. The PERL_INLINE_DIRECTORY environment variable; if set

	3. .Inline/ (in current directory); if exists and $PWD != $HOME

	4. bin.Inline (in directory of your script); if exists

	5. ~/.Inline/ - if exists

	6. ./_Inline/ - if exists

	7. bin/_Inline - if exists

	8. Create ./_Inline/ - if possible

	9. Create bin/_Inline/ - if possible

      Failing that, Inline will croak. This is rare and easily remedied by
      just making a directory that Inline will use.

      If the PERL_INSTALL_ROOT Environment Variable has been set, you will
      need to make special provision for that if the 'make install' phase
      of your Inline scripts are to succeed.

      If the module option is being compiled for permanent installation,
      then Inline will only use ./_Inline/ to build in, and the
      $Config{installsitearch} directory to install the executable in. This
      action is caused by Inline::MakeMaker, and is intended to be used in
      modules that are to be distributed on the CPAN, so that they get
      installed in the proper place.

      * Parse the Source for Semantic Cues

      Inline::C uses the module Parse::RecDescent to parse through your
      chunks of C source code and look for things that it can create
      run-time bindings to. In C it looks for all of the function
      definitions and breaks them down into names and data types. These
      elements are used to correctly bind the C function to a Perl
      subroutine. Other Inline languages like Python and Java actually use
      the python and javac modules to parse the Inline code.

      * Create the Build Environment

      Now Inline can take all of the gathered information and create an
      environment to build your source code into an executable. Without
      going into all the details, it just creates the appropriate
      directories, creates the appropriate source files including an XS
      file (for C) and a Makefile.PL.

      * Build the Code and Install the Executable

      The planets are in alignment. Now for the easy part. Inline just does
      what you would do to install a module. "`perl Makefile.PL && make &&
      make test && make install>". If something goes awry, Inline will
      croak with a message indicating where to look for more info.

      * Tidy Up

      By default, Inline will remove all of the mess created by the build
      process, assuming that everything worked. If the build fails, Inline
      will leave everything intact, so that you can debug your errors.
      Setting the noclean shortcut option will also stop Inline from
      cleaning up.

      * DynaLoad the Executable

      For C (and C++), Inline uses the DynaLoader::bootstrap method to pull
      your external module into Perl space. Now you can call all of your
      external functions like Perl subroutines.

      Other languages like Python and Java, provide their own loaders.

SEE ALSO

    For information about using Inline with C see Inline::C.

    For sample programs using Inline with C see Inline::C-Cookbook.

    For "Formerly Answered Questions" about Inline, see Inline-FAQ.

    For information on supported languages and platforms see
    Inline-Support.

    For information on writing your own Inline Language Support Module, see
    Inline-API.

    Inline's mailing list is inline@perl.org

    To subscribe, send email to inline-subscribe@perl.org

BUGS AND DEFICIENCIES

    When reporting a bug, please do the following:

      * Put "use Inline 'reportbug';" at the top of your code, or use the
      command line option "perl -MInline=reportbug ...".

      * Run your code.

      * Follow the printed directions.

AUTHOR

    Ingy döt Net <ingy@cpan.org>

    Sisyphus <sisyphus@cpan.org> fixed some bugs and is current
    co-maintainer.

COPYRIGHT

      * Copyright 2000-2015. Ingy döt Net.

      * Copyright 2008, 2010-2014. Sisyphus.

    This program is free software; you can redistribute it and/or modify it
    under the same terms as Perl itself.

    See http://www.perl.com/perl/misc/Artistic.html

## inline: Inline C, C++ and Fortran code from R

[![Build Status](https://github.com/eddelbuettel/inline/workflows/ci/badge.svg)](https://github.com/eddelbuettel/inline/actions?query=workflow%3Aci)
[![License](https://img.shields.io/badge/license-LGPL%20%28%3E%3D%202%29-brightgreen)](https://www.gnu.org/licenses/lgpl-3.0.html) 
[![CRAN](https://www.r-pkg.org/badges/version/inline)](https://cran.r-project.org/package=inline) 
[![CRAN use](https://jangorecki.gitlab.io/rdeps/inline/CRAN_usage.svg?sanitize=true)](https://cran.r-project.org/package=inline) 
[![CRAN indirect](https://jangorecki.gitlab.io/rdeps/inline/indirect_usage.svg?sanitize=true)](https://cran.r-project.org/package=inline)
[![Dependencies](https://tinyverse.netlify.app/badge/inline)](https://cran.r-project.org/package=inline)
[![Downloads](https://cranlogs.r-pkg.org/badges/inline?color=brightgreen)](https://www.r-pkg.org/pkg/inline) 
[![Debian package](https://img.shields.io/debian/v/r-cran-inline/sid?color=brightgreen)](https://packages.debian.org/sid/r-cran-inline)
[![Last Commit](https://img.shields.io/github/last-commit/eddelbuettel/inline)](https://github.com/eddelbuettel/inline)

### About

The inline package provides functionality to dynamically define R functions
(and corresponding S4 objects) from in-line C, C++ or Fortran code. It
supports the `.C`, `.Call` and `.Fortran` calling conventions.

### History

The package was originally written while Oleg Sklyar was at
[EMBL-EBI](https://www.ebi.ac.uk/).  It was then extended by Dirk
Eddelbuettel and Romain Francois for use by
[Rcpp](https://dirk.eddelbuettel.com/code/rcpp.html). Years later, Karline
Soetaert added support for Fortran. Johannes Ranke refactored some internals
and added the ability to store and retrieve compiled code.

### Authors

Oleg Sklyar, Dirk Eddelbuettel, Romain Francois, Karline Soetaert, Johannes Ranke

### Maintainer

Dirk Eddelbuettel

### License

LGPL (>= 2)

INTRODUCTION:

Inline::C - Write Perl subroutines in C.

Inline::C is the Inline Language Support Module (ILSM) for C

Example:

    use Inline C => <<'END';
    SV* JAxH(char* x) {
        return newSVpvf ("Just Another %s Hacker",x);
    }
    END
    
    print JAxH('Inline'), "\n";

When run, this complete program prints:

    Just Another Inline Hacker

-------------------------------------------------------------------------------
INSTALLATION:

This module requires the Inline module with which it is distributed. It also
requires the appropriate C compiler. (Where appropriate means the one referred
to in your Config.pm) 

Inline::C is packaged with Inline.pm. It is not necessary to install Inline::C to use Inline with some other language, like Java. 

-------------------------------------------------------------------------------
INFORMATION:

= For more infor on Inline, see 'perldoc Inline' and 'perldoc Inline-Support'
= For information about Inline::, see 'perldoc Inline::C' and
  'perldoc Inline::C-Cookbook'
= For information on writing your own Inline extension see 'perldoc Inline-API'
= For information about the Perl5 internal C API, see 'perldoc perlapi' or
  try http://www.perldoc.com/perl5.6/pod/perlapi.html

The Inline.pm mailing list is inline@perl.org. 
Send email to inline-subscribe@perl.org to subscribe.

Please send questions and comments to "Brian Ingerson" <INGY@cpan.org>

Copyright (c) 2001, 2002, Brian Ingerson. All Rights Reserved.  

NAME
    Inline::Config - Set configuration options for `Inline'

SYNOPSIS
     BEGIN {
         use Inline;
         $Inline::Config::BUILD_PREFIX = '/tmp/Inline';
         $Inline::Config::INSTALL_PREFIX = '/home/ingy/Inline';
         $Inline::Config::FORCE_BUILD = 1;
         $Inline::Config::MAKEFILE{INC} = '-I/usr/include/other');
     }

    or

     BEGIN {
         use Inline;
         Inline::Config::Build_Prefix('/tmp/Inline');
         Inline::Config::Install_Prefix('/home/ingy/Inline');
         Inline::Config::force_build(1);
         Inline::Config::makefile(INC => '-I/usr/include/other');
     }

    or

     BEGIN {
         use Inline;
         my $ic = Inline::Config->new;
         $ic->build_prefix('/tmp/Inline');
         $ic->install_prefix('/home/ingy/Inline');
         $ic->force_build(1);
         $ic->makefile(INC => '-I/usr/include/other');
     }

    or

     BEGIN {
         use Inline;
         Inline::Config->new
           ->build_prefix('/tmp/Inline')
           ->install_prefix('/home/ingy/Inline')
           ->force_build(1)
           ->makefile(INC => '-I/usr/include/other');
     }

DESCRIPTION
    `Inline::Config' is a helper module for `Inline.pm'. It sets and
    saves all of `Inline''s configuration options and defaults.

    Currently there are two categories of settings: *global* and
    *makefile*. Global settings are a collection of various options.
    See the section on "Global Settings" below. Makefile settings
    are passed directly into the Makefile.PL file. See the
    ExtUtils::MakeMaker manpage for an (long) list of available
    options.

  Methods

    All of the settings are stored as global variables in the
    `Inline::Config' namespace and are spelled with capital letters.
    You can set them in several manners depending on your personal
    style (as shown in the section on "SYNOPSIS" above).

     Direct:     $Inline::Config::BUILD_PREFIX = '/tmp/Inline';
     Function:   Inline::Config::Build_Prefix('/tmp/Inline');
     OO:         $ic->build_prefix('/tmp/Inline');
     Stacked OO: $ic->build_prefix('/tmp/Inline')->force_build(1);

    When using the direct method, the name must be all capitals.
    When using the procedural methods, any case can be used.

  Roll Your Own

    Another option is to create your own `Inline::Config.pm' module
    and set the base options there. Install the module higher up in
    the `@::INC' path. Then you don't have to set the same options
    in all of your scripts. Use the `PERL5LIB' environment variable,
    if you don't have install access to `@::INC' on your system.

  Global Settings

    This is an alphabetical list of all the global settings
    currentlly available for `Inline'. Many of them include command
    line shortcuts.

        AUTO_INCLUDE_C - Text to automatically prepend to your Inlined
        `C' code. Normally used for header files that you always
        want. Default:

         #include "EXTERN.h"
         #include "perl.h"
         #include "XSUB.h"

        BUILD_PREFIX - Tells what directory `Inline' should install the
        compiled code under. This is a base directory. "`make
        install'" actually creates a bunch of other directories
        under this. (Default is generated by examining your system
        for the best possible location. Program dies if none is
        found.)

        CLEAN_AFTER_BUILD - Tells `Inline' to remove the build files
        after compiling. You can also turn this option off from the
        command line with the special syntax "`perl -MInline=NOCLEAN
        ...'". (Default = 1)

        CLEAN_BUILD_AREA - Tells `Inline' to remove any build
        directories that may be lying around in your build area.
        Normally these directories get removed immediately after a
        successful build. Exceptions are when the build fails, or
        when you use the NOCLEAN or REPORTBUG options. You can also
        turn this option on from the command line with the special
        syntax "`perl -MInline=CLEAN ...'". (Default = 0)

        FORCE_BUILD - Forces the code to be recompiled, even if
        everything is up to date. You can also turn this option on
        from the command line with the special syntax "`perl -
        MInline=FORCE ...'". (Default = 0)

        PRINT_INFO - This is a very useful option when you want to know
        what's going on under the hood. It tells `Inline' to print
        helpful information to `STDERR'. Among the things that get
        printed is a list of which `Inline' functions were
        successfully bound to Perl. You can also set this option
        from the command line with the special syntax "`perl -
        MInline=INFO ...'". (Default = 0)

        INSTALL_LIB - Tells what directory `Inline' should add to
        `@::INC' to find your compiled code. This is based off of
        "`INSTALL_PREFIX'", so you shouldn't need to set this
        directly.

        NOTE - The code to calculate the INSTALL_LIB directory path
        is tricky, because it depends on what version of perl you
        have. If you have problems with functions not being found,
        you may need to set this yourself. Hopefully this will all
        get figured out soon.

        INSTALL_PREFIX - Tells what directory `Inline' should install
        the compiled code under. This is a base directory. "`make
        install'" actually creates a bunch of other directories
        under this. (Default is generated by examining your system
        for the best possible location. Program dies if none is
        found.)

        REPORTBUG - Puts Inline into 'REPORTBUG' mode, which does
        special processing when you want to report a bug. REPORTBUG
        also automatically forces a build, and doesn't clean up
        afterwards. This is so that you can tar and mail the build
        directory to me. REPORTBUG will print exact instructions on
        what to do. You can also set this option from the command
        line with the special syntax "`perl -MInline=REPORTBUG
        ...'". (Default = 0)

        SITE_INSTALL - Says that compiled code should be installed in
        the Perl installation's "`site_perl'" directory. Use this to
        permanently install an Inlined module. You can also set this
        option from the command line with the special syntax "`perl
        -MInline=SITE_INSTALL ...'". (Default = 0)

        TEMP_DIR - The temporary directory that "`BUILD_PREFIX'" and
        "`INSTALL_PREFIX'" are based off of. Setting this one, in
        effect, sets the other two. This directory must exist and be
        writable or `Inline' will croak.

        If you do not set this, then `Inline' will attempt to set a
        reasonable default for you. See the section on "Find a Place
        to Build and Install" in the Inline manpage for more info.

AUTHOR
    Brian Ingerson <INGY@cpan.org>

COPYRIGHT
    Copyright (c) 2000, Brian Ingerson. All Rights Reserved. This
    module is free software. It may be used, redistributed and/or
    modified under the terms of the Perl Artistic License (see
    http://www.perl.com/perl/misc/Artistic.html)

# - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
## CFunc is an S4 class derived from 'function'. This inheritance allows objects
## to behave exactly as functions do, but it provides a slot @code that keeps the
## source C or Fortran code used to create the inline call
setClass("CFunc",
  representation(
    code="character"
  ),
  contains="function"
)

setClass( "CFuncList", contains = "list" )

# - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
cfunction <- function(sig=character(), body=character(), includes=character(), otherdefs=character(),
                      language=c("C++", "C", "Fortran", "F95", "ObjectiveC", "ObjectiveC++"),
                      verbose=FALSE, convention=c(".Call", ".C", ".Fortran"), Rcpp=FALSE,
                      cppargs=character(), cxxargs=character(), libargs=character()) {

  convention <- match.arg(convention)

  if ( missing(language) ) language <- ifelse(convention == ".Fortran", "Fortran", "C++")
  else language <- match.arg(language)

  language <- switch(EXPR=tolower(language), cpp="C++", f="Fortran", f95="F95",
                     objc="ObjectiveC", objcpp= ,"objc++"="ObjectiveC++", language)

  f <- basename(tempfile())

  if ( !is.list(sig) ) {
    sig <- list(sig)
    names(sig) <- f
    names(body) <- f
  }
  if( length(sig) != length(body) )
    stop("mismatch between the number of functions declared in 'sig' and the number of function bodies provided in 'body'")

  if (Rcpp) {
      if (!require(Rcpp)) stop("Rcpp cannot be loaded, install it or use the default Rcpp=FALSE")
      cxxargs <- c(Rcpp:::RcppCxxFlags(), cxxargs)	# prepend information from Rcpp
      libargs <- c(Rcpp:::RcppLdFlags(), libargs)	# prepend information from Rcpp
  }
  if (length(cppargs) != 0) {
      args <- paste(cppargs, collapse=" ")
      if (verbose) cat("Setting PKG_CPPFLAGS to", args, "\n")
      Sys.setenv(PKG_CPPFLAGS=args)
  }
  if (length(cxxargs) != 0) {
      args <- paste(cxxargs, collapse=" ")
      if (verbose) cat("Setting PKG_CXXFLAGS to", args, "\n")
      Sys.setenv(PKG_CXXFLAGS=args)
  }
  if (length(libargs) != 0) {
      args <- paste(libargs, collapse=" ")
      if (verbose) cat("Setting PKG_LIBS to", args, "\n")
      Sys.setenv(PKG_LIBS=args)
  }
  types <- vector(mode="list", length=length(sig))
  ## GENERATE THE CODE
  for ( i in seq_along(sig) ) {
    ## C/C++ with .Call convention *********************************************
    if ( convention == ".Call" ) {
  	  ## include R includes, also error
  	  if (i == 1) {
	      code <- ifelse(Rcpp,
                         "#include <Rcpp.h>\n",
                         paste("#include <R.h>\n#include <Rdefines.h>\n",
                               "#include <R_ext/Error.h>\n", sep=""));
	      ## include further includes
	      code <- paste(c(code, includes, ""), collapse="\n")
	      ## include further definitions
	      code <- paste(c(code, otherdefs, ""), collapse="\n")
      }
  	  ## generate C-function sig from the original sig
  	  if ( length(sig[[i]]) > 0 ) {
  	    funCsig <- paste("SEXP", names(sig[[i]]), collapse=", " )
  	  }
  	  else funCsig <- ""
  	  funCsig <- paste("SEXP", names(sig)[i], "(", funCsig, ")", sep=" ")
  	  ## add C export of the function
  	  if ( language == "C++" || language == "ObjectiveC++")
  	    code <- paste( code, "extern \"C\" {\n  ", funCsig, ";\n}\n\n", sep="")
  	  ## OPEN function
  	  code <- paste( code, funCsig, " {\n", sep="")
  	  ## add code, split lines
  	  code <- paste( code, paste(body[[i]], collapse="\n"), sep="")
  	  ## CLOSE function, add return and warning in case the user forgot it
  	  code <- paste(code, "\n  ",
                    ifelse(Rcpp, "Rf_warning", "warning"),
                    "(\"your C program does not return anything!\");\n  return R_NilValue;\n}\n", sep="");
    }

    ## C/C++ with .C convention ************************************************
    else if ( convention == ".C" ) {
  	  if (i == 1) {
	      ## include only basic R includes
	      code <- ifelse(Rcpp,"#include <Rcpp.h>\n", "#include <R.h>\n")
	      ## include further includes
	      code <- paste(c(code, includes, ""), collapse="\n")
	      ## include further definitions
	      code <- paste(c(code, otherdefs, ""), collapse="\n")
      }
  	  ## determine function header
  	  if ( length(sig[[i]]) > 0 ) {
  	    types[[i]] <- pmatch(sig[[i]], c("logical", "integer", "double", "complex",
  	                       "character", "raw", "numeric"), duplicates.ok = TRUE)
  	    if ( any(is.na(types[[i]])) ) stop( paste("Unrecognized type", sig[[i]][is.na(types[[i]])]) )
  	    decls <- c("int *", "int *", "double *", "Rcomplex *", "char **",
  	               "unsigned char *", "double *")[ types[[i]] ]
  	    funCsig <- paste(decls, names(sig[[i]]), collapse=", ")
	    }
	    else funCsig <- ""
  	  funCsig <- paste("void", names(sig)[i], "(", funCsig, ")", sep=" ")
	    if ( language == "C++" || language == "ObjectiveC++" )
	      code <- paste( code, "extern \"C\" {\n  ", funCsig, ";\n}\n\n", sep="")
  	  ## OPEN function
  	  code <- paste( code, funCsig, " {\n", sep="")
  	  ## add code, split lines
  	  code <- paste( code, paste(body[[i]], collapse="\n"), sep="")
  	  ## CLOSE function
  	  code <- paste( code, "\n}\n", sep="")
    }
    ## .Fortran convention *****************************************************
    else {
  	  if (i == 1) {
	      ## no default includes, include further includes
	      code <- paste(includes, collapse="\n")
	      ## include further definitions
	      code <- paste(c(code, otherdefs, ""), collapse="\n")
      }
  	  ## determine function header
  	  if ( length(sig[[i]]) > 0 ) {
  	    types[[i]] <- pmatch(sig[[i]], c("logical", "integer", "double", "complex",
  	                       "character", "raw", "numeric"), duplicates.ok = TRUE)
  	    if ( any(is.na(types[[i]])) ) stop( paste("Unrecognized type", sig[[i]][is.na(types[[i]])]) )
  	    if (6 %in% types[[i]]) stop( "raw type unsupported by .Fortran()" )
  	    decls <- c("INTEGER", "INTEGER", "DOUBLE PRECISION", "DOUBLE COMPLEX",
  	               "CHARACTER*255", "Unsupported", "DOUBLE PRECISION")[ types[[i]] ]
  	    decls <- paste("      ", decls, " ", names(sig[[i]]), "(*)", sep="", collapse="\n")
  	    funCsig <- paste(names(sig[[i]]), collapse=", ")
  	  }
  	  else {
	      decls <- ""
	      funCsig <- ""
	    }
  	  funCsig <- paste("      SUBROUTINE", names(sig)[i], "(", funCsig, ")\n", sep=" ")
  	  ## OPEN function
  	  code <- paste( code, funCsig, decls, collapse="\n")
  	  ## add code, split lines
  	  code <- paste( code, paste(body[[i]], collapse="\n"), sep="")
  	  ## CLOSE function
  	  code <- paste( code, "\n      RETURN\n      END\n", sep="")
    }
  } ## for along signatures

  ## WRITE AND COMPILE THE CODE
  libLFile <- compileCode(f, code, language, verbose)

  ## SET A FINALIZER TO PERFORM CLEANUP
  cleanup <- function(env) {
    if ( f %in% names(getLoadedDLLs()) ) dyn.unload(libLFile)
    unlink(libLFile)
  }
  reg.finalizer(environment(), cleanup, onexit=TRUE)

  res <- vector("list", length(sig))
  names(res) <- names(sig)

  ## GENERATE R FUNCTIONS
  for ( i in seq_along(sig) ) {
    ## Create new objects of class CFunc, each containing the code of ALL inline
    ## functions. This will be used to recompile the whole shared lib when needed
    res[[i]] <- new("CFunc", code = code)

    ## this is the skeleton of the function, the external call is added below using 'body'
    ## important here: all variables are kept in the local environment
    fn <- function(arg) {
   	  NULL
    }

    DLL <- dyn.load( libLFile )

    ## Modify the function formals to give the right argument list
    args <- formals(fn)[ rep(1, length(sig[[i]])) ]
    names(args) <- names(sig[[i]])
    formals(fn) <- args

    ## create .C/.Call function call that will be added to 'fn'
    if (convention == ".Call") {
      body <- quote( CONVENTION("EXTERNALNAME", ARG) )[ c(1:2, rep(3, length(sig[[i]]))) ]
      for ( j in seq(along = sig[[i]]) ) body[[j+2]] <- as.name(names(sig[[i]])[j])
    }
    else {
      body <- quote( CONVENTION("EXTERNALNAME", as.logical(ARG), as.integer(ARG),
                    as.double(ARG), as.complex(ARG), as.character(ARG),
          			    as.raw(ARG), as.double(ARG)) )[ c(1:2,types[[i]]+2) ]
      names(body) <- c( NA, "", names(sig[[i]]) )
      for ( j in seq(along = sig[[i]]) ) body[[j+2]][[2]] <- as.name(names(sig[[i]])[j])
## OLD VERSION -- does not work for lists of functions
#      body <- quote( CONVENTION("EXTERNALNAME", as.logical(ARG), as.integer(ARG),
#                    as.double(ARG), as.complex(ARG), as.character(ARG),
#          			    as.raw(ARG), as.double(ARG)) )[ c(1:2,types+2) ]
#      names(body) <- c( NA, "", names(sig[[i]]) )
#      for ( j in seq(along = sig[[i]]) ) body[[j+2]][[2]] <- as.name(names(sig[[i]])[j])
    }
    body[[1]] <- get(convention)
    body[[2]] <- getNativeSymbolInfo( names(sig)[i], DLL )$address
    ## update the body of 'fn'
    body(fn) <- body
    ## set fn as THE function in CFunc of res[[i]]
    res[[i]]@.Data <- fn
  }

  ## OUTPUT PROGRAM CODE IF DESIRED
  if ( verbose ) {
    cat("Program source:\n")
    lines <- strsplit(code, "\n")
    for ( i in 1:length(lines[[1]]) )
      cat(format(i,width=3), ": ", lines[[1]][i], "\n", sep="")
  }

  ## Remove unnecessary objects from the local environment
  remove(list = c("args", "body", "fn", "funCsig", "i", "includes", "j"))

  ## RETURN THE FUNCTION
  if (length(res) == 1 && names(res) == f) return( res[[1]] )
  else return( new( "CFuncList", res ) )
}

# - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
compileCode <- function(f, code, language, verbose) {
  wd = getwd()
  on.exit(setwd(wd))
  ## Prepare temp file names
  if ( .Platform$OS.type == "windows" ) {
    ## windows files
    dir <- gsub("\\\\", "/", tempdir())
    libCFile  <- paste(dir, "/", f, ".EXT", sep="")
    libLFile  <- paste(dir, "/", f, ".dll", sep="")
    libLFile2 <- paste(dir, "/", f, ".dll", sep="")
  }
  else {
    ## UNIX-alike build
    libCFile  <- paste(tempdir(), "/", f, ".EXT",               sep="")
    libLFile  <- paste(tempdir(), "/", f, .Platform$dynlib.ext, sep="")
    libLFile2 <- paste(tempdir(), "/", f, ".sl",                sep="")
  }
  extension <- switch(language, "C++"=".cpp", C=".c", Fortran=".f", F95=".f95",
                                ObjectiveC=".m", "ObjectiveC++"=".mm")
  libCFile <- sub(".EXT$", extension, libCFile)

  ## Write the code to the temp file for compilation
  write(code, libCFile)

  ## Compile the code using the running version of R if several available
  if ( file.exists(libLFile) ) file.remove( libLFile )
  if ( file.exists(libLFile2) ) file.remove( libLFile2 )

  setwd(dirname(libCFile))
  errfile <- paste( basename(libCFile), ".err.txt", sep = "" )
  cmd <- paste(R.home(component="bin"), "/R CMD SHLIB ", basename(libCFile), " 2> ", errfile, sep="")
  if (verbose) cat("Compilation argument:\n", cmd, "\n")
  compiled <- system(cmd, intern=!verbose)
  errmsg <- readLines( errfile )
  unlink( errfile )
  writeLines( errmsg )
  setwd(wd)

  if ( !file.exists(libLFile) && file.exists(libLFile2) ) libLFile <- libLFile2
  if ( !file.exists(libLFile) ) {
    cat("\nERROR(s) during compilation: source code errors or compiler configuration errors!\n")
    cat("\nProgram source:\n")
    code <- strsplit(code, "\n")
    for (i in 1:length(code[[1]])) cat(format(i,width=3), ": ", code[[1]][i], "\n", sep="")
    stop( paste( "Compilation ERROR, function(s)/method(s) not created!", paste( errmsg , collapse = "\n" ) ) )
  }
  return( libLFile )
}

library(inline)

isSolaris <- Sys.info()[["sysname"]] == "SunOS"

n <- 10L
x <- 1:10

## A simple Fortran example - n and x: assumed-size vector
code <- "
      integer i
      do 1 i=1, n(1)
    1 x(i) = x(i)**3
"

cubefn <- cfunction(signature(n = "integer", x = "numeric"), code,
  convention = ".Fortran")

res_cube <- list(
  n = 10L,
  x = c(1, 8, 27, 64, 125, 216, 343, 512, 729, 1000))

res_1 <- cubefn(n, x)
expect_identical(res_cube, res_1)

cubefn_named <- cfunction(signature(n = "integer", x = "numeric"), code,
  convention = ".Fortran", name = "cubefn")
expect_identical(cubefn_named(n, x), res_1)

expect_true(grepl("cubefn", cubefn_named@code))

## Same Fortran example - now n is one number
code2 <- "
      integer i
      do 1 i=1, n
    1 x(i) = x(i)**3
"
cubefn2 <- cfunction(signature(n = "integer", x = "numeric"),
  implicit = "none", dim = c("", "(*)"), code2, convention=".Fortran")

res_2 <- cubefn2(n, x)
expect_identical(res_2, res_cube)

## Same in F95, now x is fixed-size vector (length = n)
code3 <- "x = x*x*x"
cubefn3 <- cfunction(signature(n = "integer", x = "numeric"),
  implicit = "none", dim = c("", "(n)"), code3, language="F95")
res_3 <- cubefn3(n, x)
expect_identical(res_3, res_cube)

## Same example in C
code4 <- "
      int i;
      for (i = 0; i < *n; i++)
        x[i] = x[i]*x[i]*x[i];
"
cubefn4 <- cfunction(signature(n = "integer", x = "numeric"), code4,
  language = "C", convention = ".C")
res_4 <- cubefn4(n, x)
expect_identical(res_4, res_cube)

if (isSolaris) exit_file("Skip remainder")

## use of a module in F95
modct <- "module modcts
double precision, parameter :: pi = 3.14159265358979
double precision, parameter :: e = 2.71828182845905
end"

getconstants <- "x(1) = pi
x(2) = e"

cgetcts <- cfunction(body = getconstants, module = "modcts", implicit = "none",
  includes = modct, sig = c(x = "double"), dim = c("(2)"), language = "F95")

res_5 <- cgetcts(x = c(1, 2))
expect_equal(res_5$x, c(pi, exp(1)), tolerance = 1e-7)

## Use of .C convention with C code
## Defining two functions, one of which calls the other
sigSq <- signature(n = "integer", x = "numeric")
codeSq <- "
  for (int i=0; i < *n; i++) {
    x[i] = x[i]*x[i];
  }"
sigQd <- signature(n = "integer", x = "numeric")
codeQd <- "
  squarefn(n, x);
  squarefn(n, x);
"

fns <- cfunction(
  sig = list(squarefn = sigSq, quadfn = sigQd),
  body = list(codeSq, codeQd),
  convention = ".C")

res_square <- list(
  n = 10L,
  x = c(1, 4, 9, 16, 25, 36, 49, 64, 81, 100))

res_quad <- list(
  n = 10L,
  x = c(1, 16, 81, 256, 625, 1296, 2401, 4096, 6561, 10000))

res_6_square <- fns[["squarefn"]](n, x)
res_6_quad <- fns[["quadfn"]](n, x)

expect_identical(res_6_square, res_square)
expect_identical(res_6_quad, res_quad)

## Alternative declaration using 'setCMethod'
setCMethod(c("squarefn", "quadfn"), list(sigSq, sigQd),
           list(codeSq, codeQd), convention = ".C")

res_7_square <- squarefn(n, x)
res_7_quad <- quadfn(n, x)

expect_identical(res_7_square, res_square)
expect_identical(res_7_quad, res_quad)

# - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
setGeneric( "setCMethod", function(f, sig, body, ...) standardGeneric("setCMethod") )

# - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
setMethod( "setCMethod", signature(f="character", sig="list", body="list"),
  function(f, sig, body, includes="", otherdefs="",
           language=c("C++", "C", "Fortran", "F95", "ObjectiveC", "ObjectiveC++"),
           verbose=FALSE, convention=c(".Call", ".C", ".Fortran"),
           where=topenv(.GlobalEnv), ...) {
    if ( length(f) != length(sig) || length(sig) != length(body) )
      stop("number of signatures does not correspond to the number of code chunks")

    names(sig) <- f
    fns <- cfunction(sig, body, includes, otherdefs, language, verbose, convention)

    if ( verbose )
      cat("\nThe following methods are now defined:\n")
    ## Let's try to create generics
    for ( i in 1:length(f) ) {
      generic <- paste( "setGeneric(\"", f[i], "\", function(",
                        paste(names(sig[[i]]),collapse=", "), ") standardGeneric(\"",
                        f[i], "\"),where=where )", sep="")
      eval(parse(text=generic))
      setMethod(f[i], sig[[i]], fns[[i]], where=where)
      if ( verbose ) showMethods(f[i])
    }
  }
)

# - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
setMethod( "setCMethod", signature(f="character", sig="character", body="character"),
  function(f, sig, body, includes="", otherdefs="", language=c("C++", "C", "Fortran", "F95", "ObjectiveC", "ObjectiveC++"),
                      verbose=FALSE, convention=c(".Call", ".C", ".Fortran"), where=topenv(.GlobalEnv), ...)
    setCMethod(f, list(sig), list(body), includes, otherdefs, language, verbose, convention, where=topenv(.GlobalEnv), ...)
)

setGeneric("getDynLib", function(x, ...) standardGeneric("getDynLib") )

setMethod( "getDynLib", signature( x = "character" ),
function( x ){
	dlls <- getLoadedDLLs()
	if( x %in% names( dlls ) ){
		dlls[[ x ]]
	} else {
		stop( sprintf( "dll %s not loaded", x ) )
	}
} )

setMethod( "getDynLib", signature( x = "CFunc" ),
function( x ){
	env <- environment( x@.Data )
	f <- get( "f", env )
	dlls <- getLoadedDLLs()
	dll <- if( ! f %in% names(dlls) ){
		dyn.load( get( "libLFile", env ) )
	} else{
		dlls[[ f ]]
	}
	dll
} )

setMethod( "getDynLib", signature( x = "CFuncList" ), function(x) getDynLib( x[[1L]] ) )

setGeneric("moveDLL",
  function(x, ...) {
    standardGeneric("moveDLL")
  }
)

setMethod("moveDLL",
  signature(x = "CFunc"),
  function(x, name, directory, unload = FALSE, overwrite = FALSE, verbose = FALSE) {

    # Check arguments
    if (length(name) > 1) stop("Please supply only one name")
    if (length(directory) > 1) stop("Please supply only one directory name")

    # Obtain path to DLL
    old_path <- environment(x)$libLFile

    # Create new path
    if (!dir.exists(directory)) stop("There is no directory ", directory)
    extension <- tools::file_ext(old_path)
    new_path <- normalizePath(file.path(directory, paste(name, extension, sep = ".")),
      mustWork = FALSE)

    active_paths <- sapply(getLoadedDLLs()[-1],
      function(di) normalizePath(di[["path"]]))
    if (new_path %in% active_paths) {
      if (unload) {
        if (inherits(try(dyn.unload(new_path)), "try-error"))
          stop("Could not unload ", new_path)
      } else {
        stop("DLL from ", new_path, " is in use")
      }
    }

    # Copy the DLL
    copy_success <- file.copy(old_path, new_path, overwrite = overwrite)
    if (!copy_success) stop("Failed to copy DLL from ", old_path, " to ", new_path)
    if (verbose) message("Copied DLL from ", old_path, " to ", new_path)

    # Adjust the path that getDynLib uses
    environment(x)$libLFile <- new_path

    invisible()
  }
)

writeCFunc <- function(x, file) {
  env <- environment(x)
  if (identical(env$libLFile, env$libLFile_orig))
    stop("Use moveDLL to prevent losing the DLL by garbage collection or session termination")

  saveRDS(x, file = file)
}

readCFunc <- function(file) {
  x <- readRDS(file)
  if (!inherits(x, "CFunc")) stop(file, " does not contain a serialized CFunc object")

  # Get code for restoring after updating the function body
  source_code <- x@code

  # Load the DLL
  env <- environment(x)
  dll_info <- dyn.load(env$libLFile)

  # Set the symbol info in the function body
  body(x)[[2]] <- getNativeSymbolInfo(env$name, dll_info[["name"]])[["address"]]
  x_cf <- as(x, "CFunc")
  x_cf@code <- source_code

  return(x_cf)
}

setGeneric("code", function(x, ...) standardGeneric("code") )
setMethod( "code", signature( x = "character" ),
function( x, linenumbers = TRUE ){
  lines <- strsplit(x, "\n")
  if (linenumbers)
   for (i in 1:length(lines[[1]])) cat(format(i, width = 3),
    ": ", lines[[1]][i], "\n", sep = "")
  else
   for (i in 1:length(lines[[1]])) cat(lines[[1]][i], "\n", sep = "")

} )
setMethod( "code", signature( x = "CFunc" ), function( x, linenumbers = TRUE  ) code (x@code, linenumbers))
setMethod( "code", signature( x = "CFuncList" ), function(x, linenumbers = TRUE ) code( x[[1L]], linenumbers ) )

## Printing methods

setMethod( "print", signature( x = "CFunc" ),
function( x ){
  cat("An object of class 'CFunc'\n")
  Dat <- x@.Data
  print(Dat)
  cat("code:\n")
  code(x)
} )

setMethod( "print", signature( x = "CFuncList" ), function(x) {
  cat("An object of class 'CFuncList'\n")
  for (i in 1:length(x)) {
    print(names(x)[i])
    print(x[[i]]@.Data )
    cat("\n")
  }
  cat("code:\n")
  code(x)
} )

library(inline)

isM1 <- grepl("aarch64-apple", R.version$platform)
isLinux <- Sys.info()[["sysname"]] == "Linux"

code <- "
      int i;
      for (i = 0; i < *n; i++)
        x[i] = x[i]*x[i];
"
quadfn <- cfunction(signature(n = "integer", x = "numeric"), code,
  language = "C", convention = ".C")

res_known <- list(n = 5L, x = c(1, 4, 9, 16, 25))
expect_identical(quadfn(5, 1:5), res_known)

# Saving and restoring the function removes the pointer to the DLL
quadfn_path <- file.path(tempdir(), "quadfn.rda")
save(quadfn, file = quadfn_path)
rm(quadfn)
load(quadfn_path)
expect_error(quadfn(5, 1:5), "NULL value passed as symbol address")

# The DLL is removed by garbage collection
gc()
expect_false(file.exists(environment(quadfn)$libLFile))

if (isM1) exit_file("Skip remainer")
if (!isLinux) exit_file("Skip remainer")

# So we recreate the function and move the DLL to a user defined location
quadfn <- cfunction(signature(n = "integer", x = "numeric"), code,
  language = "C", convention = ".C")
moveDLL(quadfn, name = "testname", directory = tempdir())
expect_identical(quadfn(5, 1:5), res_known)

expect_error(
  moveDLL(quadfn, name = "testname", directory = tempdir(), unload = TRUE),
  "Failed to copy")

expect_error(
  moveDLL(quadfn, name = "testname", directory = tempdir(), unload = TRUE,
  overwrite = TRUE),
  "file can not be copied both 'from' and 'to'")

# We recreate the function to have a new temporary DLL name
quadfn <- cfunction(signature(n = "integer", x = "numeric"), code,
  language = "C", convention = ".C")

expect_identical(quadfn(5, 1:5), res_known)

# Now the new path is taken and loaded, but we can unload and overwrite
moveDLL(quadfn, name = "testname", directory = tempdir(), unload = TRUE,
  overwrite = TRUE)
expect_identical(quadfn(5, 1:5), res_known)

# Now the DLL is not removed by garbage collection
gc()
expect_true(file.exists(environment(quadfn)$libLFile))
# But we still get the pointer removed when saving and restoring
save(quadfn, file = quadfn_path)
rm(quadfn)
load(quadfn_path)
expect_error(quadfn(5, 1:5), "NULL value passed as symbol address")

# So we recreate the function again, move the DLL, write and restore
quadfn <- cfunction(signature(n = "integer", x = "numeric"), code,
  language = "C", convention = ".C")
moveDLL(quadfn, name = "testname", directory = tempdir(), unload = TRUE,
  overwrite = TRUE)
writeCFunc(quadfn, quadfn_path)
quadfn_reloaded <- readCFunc(quadfn_path)
expect_identical(quadfn_reloaded(5, 1:5), res_known)


# Create a function with a user defined function name in the source code,
# save and restore
quadfn_named <- cfunction(signature(n = "integer", x = "numeric"), code,
  language = "C", convention = ".C", name = "quadfn")
moveDLL(quadfn_named, name = "quadfn_dll", directory = tempdir(), unload = TRUE,
  overwrite = TRUE)
writeCFunc(quadfn_named, quadfn_path)
quadfn_named_reloaded <- readCFunc(quadfn_path)
expect_identical(quadfn_named_reloaded(5, 1:5), res_known)
expect_true(grepl("quadfn", quadfn_named_reloaded@code))

setGeneric( "package.skeleton" )

setMethod( "package.skeleton", signature( name = "character", list = "CFuncList" ),
function (name = "anRpackage", list = new( "CFuncList" ), environment = .GlobalEnv,
    path = ".", force = FALSE, code_files = character()) {

	env <- environment( list[[1]]@.Data )
	clean.env <- new.env( parent = environment )

	functions <- names( list )
	for( i in seq_along(list) ){
		f <- functions[i]
		call <- body( list[[i]]@.Data )
		call[[1L]] <- as.name( env$convention )
		call[[2L]] <- names( env$sig )[i]
		call[["PACKAGE"]] <- name
		fun <- list[[i]]@.Data
		body( fun ) <- call
		environment(fun) <- clean.env
		assign( f, fun, clean.env )
	}

	message( ">> standard package.skeleton from utils" )
	package.skeleton( name, functions , clean.env, path, force, code_files )

	if( !file.exists( R <- file.path(name, "R") ) ){
		dir.create( R )
		con <- file( file.path( R, sprintf("%s.R", name) ), open = "w" )
		for( i in seq_along(list) ){
			fun <- functions[i]
			writeLines( sprintf( ' %s <- %s\n' , fun, paste( deparse(clean.env[[ fun ]] ), collapse = "\n" ) ), con )
		}
		close( con )
		message( ">> added R directory with function calling compiled code" )
	}

	Rdfiles <- file.path( name, "man", sprintf( "%s.Rd", functions ) )
	sapply( Rdfiles, function( Rdfile) {
		if( file.exists( Rdfile ) ){
			content <- readLines( Rdfile )
			content <- sub( "%%  ~~function to do ... ~~", "insert here the title", content )
			writeLines( content, Rdfile )
		}
		message( ">> workaround for empty title in Rd file" )
	} )

	package.doc <- file.path( name, "man", sprintf("%s-package.Rd", name) )
	if( file.exists( package.doc) ){
		lines <- readLines( package.doc )
		lines <- sub( "~~ simple examples of", "%% ~~ simple examples of", lines )
		writeLines( lines, package.doc )
	}

	dir.create( file.path( name, "src" ) )
	message( ">> created src directory" )

	NAMESPACE <- sprintf( '
	useDynLib(%s)

	%s
	', name, paste( sprintf( 'export("%s")', functions ), collapse = "\n" ) )
	writeLines( NAMESPACE, file.path( name, "NAMESPACE" ) )

	language <- env$language
	extension <- switch(language, "C++"=".cpp", C=".c", Fortran=".f", F95=".f95",
                                ObjectiveC=".m", "ObjectiveC++"=".mm")

	cfile <- file.path( name, "src", sprintf( "%s%s", name, extension ) )
	writeLines( list[[1]]@code, cfile )
	message( ">> added compiled code in src directory" )

	settings <- env$settings
	if( !is.null( settings ) ){
		DESCRIPTION_file <- file.path( name, "DESCRIPTION" )
		DESCRIPTION <- read.dcf( DESCRIPTION_file )

		depends <- settings$Depends
		if( !is.null( depends ) ){
			DESCRIPTION <- cbind( DESCRIPTION, "Depends" = paste( depends, collapse = ", ") )
		}

		linkingTo <- settings$LinkingTo
		if( !is.null( linkingTo ) ){
			DESCRIPTION <- cbind( DESCRIPTION, "LinkingTo" = paste( linkingTo, collapse = ", ") )
		}
		write.dcf( DESCRIPTION, DESCRIPTION_file )

		message( ">> updated DESCRIPTION file" )

		Makevars <- settings$Makevars
		if( !is.null( Makevars ) ){
			Makevars_file <- file.path( name, "src", "Makevars" )
			writeLines( Makevars, Makevars_file )
			message( ">> added Makevars ")
		}

		Makevars.win <- settings$Makevars.win
		if( !is.null( Makevars.win ) ){
			Makevars.win_file <- file.path( name, "src", "Makevars.win" )
			writeLines( Makevars.win, Makevars.win_file )
			message( ">> added Makevars.win" )
		}

		configure <- settings$configure
		if( ! is.null( configure ) ){
		    configure_file <- file.path(name, "configure" )
		    writeLines( configure, configure_file )
                    Sys.chmod(configure_file, "0755")
		    message( ">> added configure" )
		}

		configure.win <- settings$configure.win
		if( ! is.null( configure.win ) ){
		    configure_win_file <- file.path(name, "configure.win" )
		    writeLines( configure.win, configure_win_file )
		    message( ">> added configure.win" )
		}

		Makevars.in <- settings$Makevars.in
		if( ! is.null( Makevars.in ) ){
		    Makevars_in_file <- file.path(name, "src", "Makevars.in" )
		    writeLines( Makevars.in, Makevars_in_file )
		    message( ">> added Makevars.in" )
		}

	}
	invisible(NULL)

} )


setMethod( "package.skeleton", signature( name = "character", list = "CFunc" ),
function (name = "anRpackage", list = new( "CFunc" ), environment = .GlobalEnv,
    path = ".", force = FALSE, code_files = character() ) {

    funclist <- new( "CFuncList", base::list( fun = list ) )
    package.skeleton( name = name, list = funclist,
		environment = environment, path = path , force = force,
		code_files = code_files )
} )

plugins <- new.env()

plugins[["default"]] <- function() {
    includes = '
#include <R.h>
#include <Rdefines.h>
#include <R_ext/Error.h>
'
    list(includes = includes,
         body = function(x) {
             paste0(x,
                    '\nRf_warning("your C++ program does not return anything");',
                    '\nreturn R_NilValue;')})
}

registerPlugin <- function(name, plugin) {
    plugins[[ name ]] <- plugin
}

getPlugin <- function(name, ...) {
    if (name %in% ls(plugins)) {
        plugins[[ name ]](...)
    } else if (sprintf("package:%s", name) %in% search() || requireNamespace(name, quietly = TRUE)) {
        plugin <- get("inlineCxxPlugin", asNamespace(name))
        if (is.null(plugin)) {
            stop(sprintf("package '%s' does not define an inline plugin", name))
        }
        registerPlugin(name, plugin)
        plugin(...)
    } else {
        stop(sprintf("could not find plugin '%s'", name))
    }
}

paste0 <- function(...) paste(..., sep="")

addLineNumbers <- function(code) {
    code <- strsplit(paste(code, collapse = "\n" ), "\n")[[1]]
    sprintf("%4d : %s", 1:length(code), code)
}

cxxfunction <- function(sig = character(), body = character(),
                        plugin = "default", includes = "",
                        settings = getPlugin(plugin),
                        ..., verbose = FALSE) {
    f <- basename(tempfile())

    if (!is.list(sig)) {
        sig <- list(sig)
        names(sig) <- f
        if (!length(body)) body <- ""
        names(body) <- f
    }
    if (length(sig) != length(body))
        stop("mismatch between the number of functions declared in 'sig' and the number of function bodies provided in 'body'")

    signature <- lapply( sig, function(x) {
        if (!length(x)){
            ""
        } else {
            paste(sprintf("SEXP %s", names(x) ), collapse = ", ")
        }
    })

    decl <- lapply(1:length(sig) , function(index) {
        sprintf('SEXP %s( %s) ;', names(signature)[index], signature[[index]])
    } )

    def <- lapply(1:length(sig), function(index){
        sprintf('SEXP %s(%s) {\n%s\n}',
                names(signature)[index],
                signature[[index]],
                if (is.null(settings$body)) body[[index]] else settings$body(body[[index]]) )
    })

    settings_includes <- if (is.null(settings$includes)) "" else paste(settings$includes, collapse = "\n")

    code <- sprintf('
// includes from the plugin
%s

// user includes
%s

// declarations
extern "C" {
%s
}

// definition
%s
',
                     settings_includes,
                     paste(includes, collapse = "\n"),
                     paste(decl, collapse = "\n"),
                     paste(def, collapse = "\n"))


    if (!is.null(env <- settings$env)) {
        do.call(Sys.setenv, env)
        if (isTRUE(verbose)) {
            cat(" >> setting environment variables: \n")
            writeLines(sprintf("%s = %s", names(env), env))
        }
    }

    LinkingTo <- settings$LinkingTo
    if (!is.null(LinkingTo)) {
        paths <- find.package(LinkingTo, quiet=TRUE)
        if (length(paths)) {
            flag <- paste(paste0('-I"', paths, '/include"'), collapse = " ")
            Sys.setenv(CLINK_CPPFLAGS = flag)
            if (isTRUE(verbose)) {
                cat(sprintf("\n >> LinkingTo : %s\n", paste(LinkingTo, collapse = ", ")))
                cat("CLINK_CPPFLAGS = ", flag, "\n\n")
            }
        }

    }

    if (isTRUE(verbose)) {
        writeLines(" >> Program source :\n")
        writeLines(addLineNumbers(code))
    }

    language <- "C++"

    ## WRITE AND COMPILE THE CODE
    libLFile <- compileCode(f, code, language = language, verbose = verbose)

    ## SET A FINALIZER TO PERFORM CLEANUP
    cleanup <- function(env) {
        if (f %in% names(getLoadedDLLs())) dyn.unload(libLFile)
        unlink(libLFile)
    }
    reg.finalizer(environment(), cleanup, onexit=TRUE)

    ## Create new objects of class CFunc, each containing the code of ALL inline
    ## functions. This will be used to recompile the whole shared lib when needed
    res <- vector("list", length(sig))
    names(res) <- names(sig)
    res <- new("CFuncList", res)

    DLL <- dyn.load(libLFile)

    for (i in seq_along(sig)) {
        res[[i]] <- new("CFunc", code = code)

        fn <- function(arg) {
            NULL
        }

    	## Modify the function formals to give the right argument list
    	args <- formals(fn)[ rep(1, length(sig[[i]])) ]
    	names(args) <- names(sig[[i]])
    	formals(fn) <- args

    	## create .Call function call that will be added to 'fn'
        body <- quote(.Call(EXTERNALNAME, ARG))[ c(1:2, rep(3, length(sig[[i]]))) ]
        for (j in seq_along(sig[[i]])) body[[j+2]] <- as.name(names(sig[[i]])[j])

        body[[1L]] <- .Call
        body[[2L]] <- getNativeSymbolInfo(names(sig)[[i]], DLL)$address
        ## update the body of 'fn'
        body(fn) <- body
        ## set fn as THE function in CFunc of res[[i]]
        res[[i]]@.Data <- fn
    }

    ## clear the environment
    rm(j)
    convention <- ".Call"
    if (identical(length(sig), 1L)) res[[1L]] else res
}

rcpp <- function(..., plugin="Rcpp") cxxfunction(..., plugin=plugin)

library(inline)

## basic examples from manual page
fx <- cxxfunction(signature(x = "integer", y = "numeric"),
                  "return Rf_ScalarReal(INTEGER(x)[0] * REAL(y)[0]);")
expect_true(is(fx, "CFunc"))
expect_equal(fx(2L, 5), 10)

if (!requireNamespace("Rcpp", quietly=TRUE)) exit_file("Need Rcpp for remainder of tests")

fx <- cxxfunction(signature(x = "integer", y = "numeric"),
                  "return wrap(as<int>(x) * as<double>(y));",
                  plugin = "Rcpp")
expect_true(is(fx, "CFunc"))
expect_equal(fx(2L, 5), 10)

## equivalent shorter form using rcpp()
fx <- rcpp(signature(x = "integer", y = "numeric"),
           "return wrap(as<int>(x) * as<double>(y));")
expect_true(is(fx, "CFunc"))
expect_equal(fx(2L, 5), 10)

if (requireNamespace("tinytest", quietly=TRUE) &&
  utils::packageVersion("tinytest") >= "1.1.0") {

  ## Set a seed to make the tests deterministic
  set.seed(42)

  tinytest::test_package("inline")
}

#!/usr/bin/env bash

set -e

export PERL5LIB=$PWD/lib

ILSM=(
  ingydotnet/inline-c-pm
  daoswald/Inline-CPP
)

main() {
  for repo in ${ILSM[@]}; do
    echo "Testing '$repo'"
    check-repo || continue
    (
      set -x
      cd "$repo_path"
      if [ -e Meta ]; then
        make test
        make cpantest
        make disttest
      else
        perl Makefile.PL < /dev/null
        make test
        make purge
      fi
    )
  done
  echo "All ILSM tests passed!"
}

check-repo() {
  repo_path="test/devel/repo/${repo#*/}"
  if [ ! -e "$repo_path" ]; then
    mkdir -p test/devel
    git clone "git@github.com:$repo" "$repo_path"
  else
    (
      set -x
      cd "$repo_path"
      branch="$(git rev-parse --abbrev-ref HEAD)"
      if [ "$branch" != master ]; then
        echo "Repo '$repo_path' is not on master"
      fi
      git reset --hard
      git clean -fxd
      if [ -n "$(git status -s)" ]; then
        die "Repo '$repo_path' is not clean"
      fi
      git pull --rebase origin master
    ) || return 1
  fi
  return 0
}

die() {
  echo "$@"
  exit 1
}

[ "$BASH_SOURCE" == "$0" ] && main "$@"

:

\name{cfunction}

\alias{cfunction}
\alias{setCMethod}

\concept{inline function call}

\title{ Inline C, C++, Fortran function calls from R }

\description{
  Functionality to dynamically define R functions and S4 methods with in-lined C, 
  C++ or Fortran code supporting .C and .Call calling conventions.
}

\usage{
  cfunction(sig=character(), body=character(), includes=character(),
            otherdefs=character(), 
            language=c("C++", "C", "Fortran", "F95", "ObjectiveC", "ObjectiveC++"),
            verbose=FALSE, 
            convention=c(".Call", ".C", ".Fortran"),
            Rcpp=FALSE,
            cppargs=character(), cxxargs=character(), libargs=character())

  ## S4 methods for signatures
  #  f='character', sig='list', body='list'
  #  f='character', sig='character', body='character'
  
  setCMethod(f, sig, body, ...)

  ## Further arguments:
  #  setCMethod(f, sig, body, includes="", otherdefs="", cpp=TRUE,
  #  verbose=FALSE, where=topenv(.GlobalEnv), ...)
}

\arguments{
  \item{f}{A single character value if \code{sig} and \code{body} are character vectors
    or a character vector of the same length and the length of \code{sig} or
    \code{body} with the name(s) of methods to create.}
    
  \item{sig}{A match of formal argument names for the function with the
    character-string names of corresponding classes. Alternatively, 
    a list of such character vectors. }

  \item{body}{ A character vector with C, C++ or Fortran code omitting function
    declaration (only the body, i.e. in case of C starting after the function 
    opening curly bracket and ending before the closing curly bracket, 
    brackets excluded). In case of \code{setCMethod} with signature
    \code{list} -- a list of such character vectors. } 

  \item{includes}{ A character vector of additional includes and preprocessor
    statements etc that will be put between the R includes and the user function(s).}

  \item{otherdefs}{ A characted vector with the code for any further definitions of 
    functions, classes, types, forward declarations, namespace usage clauses etc 
    which is inserted between the includes and the declarations of the functions
    defined in \code{sig}.}

  \item{language}{ A character value that specifies the source language of the
    inline code. The possible values for \code{language} include all those 
    supported by \code{R CMD SHLIB} on any platform, which are currently C, 
    C++, Fortran, F95, ObjectiveC and ObjectiveC++; they may not all be supported
    on your platform. One can specify the language either in full as above, or 
    using any of the following case insensitive shortened forms: \code{c, cpp, 
    c++, f, f95, objc, objcpp, objc++}. Defaults to \code{C++}.}
    
  \item{verbose}{ If \code{TRUE} prints the compilation output, the source
    code of the resulting program and the definitions of all declared
    methods. If \code{FALSE}, the function is silent, but it prints compiler
    warning and error messages and the source code if compilation fails. }
    
  \item{convention}{ Which calling convention to use?  See the Details section.}

  \item{Rcpp}{If \code{TRUE} adds inclusion of \code{Rcpp.h} to
  \code{includes}, also queries the \code{Rcpp} package about
  the location of header and library files and sets environment
  variables \code{PKG_CXXFLAGS} and \code{PKG_LIBS} accordingly so that
  the R / C++ interface provided by the \code{Rcpp} package can be
  used. Default value is \code{FALSE}.}

  \item{cppargs}{Optional character vector of tokens to be passed to
  the compiler via the \code{PKG_CPPFLAGS} environment
  variable. Elements should be fully formed as for example
  \code{c("-I/usr/local/lib/foo", "-DDEBUG")} and are passed along verbatim.}

  \item{cxxargs}{Optional character vector of tokens to be passed to
  the compiler via the \code{PKG_CXXFLAGS} environment
  variable. Elements should be fully formed as for example
  \code{c("-I/usr/local/lib/foo", "-DDEBUG")} and are passed along verbatim.}

  \item{libargs}{Optional character vector of tokens to be passed to the
  compiler via the \code{PKG_LIBS} environment variable. Elements should
  be fully formed as for example \code{c("-L/usr/local/lib/foo -lfoo",
  "--lpthread")} and are passed along verbatim.}
    
  \item{...}{ Reserved.}
}

\value{
  If \code{sig} is a single character vector, \code{cfunction} returns a single
  \code{\link{function}}; if it is a list, it returns a list of functions.
  
  \code{setCMethod} declares new methods with given names and signatures and
  returns invisible \code{NULL}.
}

\details{
  
  To declare multiple functions in the same library one can use \code{setCMethod}
  supplying lists of signatures and implementations. In this case, provide as
  many method names in \code{f} as you define methods. Avoid clashes when selecting
  names of the methods to declare, i.e. if you provide the same name several times
  you must ensure that signatures are different but can share the same generic!

  The source code in the \code{body} should not include the header or
  "front-matter" of the function or the close, e.g. in C or C++ it
  must start after the C-function opening curly bracket and end before
  the C-function closing curly bracket, brackets should not be
  included. The header will be automatically generated from the R-\code{signature} 
  argument. Arguments will will carry the same name as used in the signature, 
  so avoid variable names that are not legal in the target language 
  (e.g. names with dots).
  
  C/C++: If \code{convention == ".Call"} (the default), the \code{\link{.Call}} mechanism 
  is used and its result is returned directly as the result of the call of the 
  generated function.  As the last line of the generated C/C++ code a 
  \code{return R_NilValue;} is added in this case and a warning is generated 
  in case the user has forgotten to provide a return value. To suppress the 
  warning and still return NULL, add \code{return R_NilValue;} explicitly.
  
  Special care is needed with types, memory allocation and protection
  -- exactly the same as if the code was not inline: see the
  Writing R Extension manual for information on \code{\link{.Call}}.  
  
  If \code{convention == ".C"} or \code{convention == ".Fortran"}, the
  \code{\link{.C}} or \code{\link{.Fortran}} mechanism respectively is
  used, and the return value is a list containing all arguments.
  
  Attached R includes include \code{R.h} for \code{".C"}, and
  additionally \code{Rdefines.h} and \code{R_ext\\Error.h} for
  \code{".Call"}.
}

\seealso{ \code{
    \link{Foreign} Function Interface
}}

\examples{

x <- as.numeric(1:10)
n <- as.integer(10)

\dontrun{
## A simple Fortran example
code <- "
      integer i
      do 1 i=1, n(1)
    1 x(i) = x(i)**3
"
cubefn <- cfunction(signature(n="integer", x="numeric"), code, convention=".Fortran")

cubefn(n, x)$x
}


## Use of .C convention with C code
## Defining two functions, one of which calls the other
sigSq <- signature(n="integer", x="numeric")
codeSq <- "
  for (int i=0; i < *n; i++) {
    x[i] = x[i]*x[i];
  }"
sigQd <- signature(n="integer", x="numeric")
codeQd <- "
  squarefn(n, x);
  squarefn(n, x);
"

fns <- cfunction( list(squarefn=sigSq, quadfn=sigQd), 
                  list(codeSq, codeQd), 
                  convention=".C")

squarefn <- fns[["squarefn"]]
quadfn <- fns[["quadfn"]]

squarefn(n, x)$x
quadfn(n, x)$x

## Alternative declaration using 'setCMethod'
setCMethod(c("squarefn", "quadfn"), list(sigSq, sigQd), 
           list(codeSq, codeQd), convention=".C")
           
squarefn(n, x)$x
quadfn(n, x)$x

## Use of .Call convention with C code
## Multyplying each image in a stack with a 2D Gaussian at a given position
code <- "
  SEXP res;
  int nprotect = 0, nx, ny, nz, x, y;
  PROTECT(res = Rf_duplicate(a)); nprotect++;
  nx = INTEGER(GET_DIM(a))[0];
  ny = INTEGER(GET_DIM(a))[1];
  nz = INTEGER(GET_DIM(a))[2];
  double sigma2 = REAL(s)[0] * REAL(s)[0], d2 ;
  double cx = REAL(centre)[0], cy = REAL(centre)[1], *data, *rdata;
  for (int im = 0; im < nz; im++) {
    data = &(REAL(a)[im*nx*ny]); rdata = &(REAL(res)[im*nx*ny]);
    for (x = 0; x < nx; x++)
      for (y = 0; y < ny; y++) {
        d2 = (x-cx)*(x-cx) + (y-cy)*(y-cy);
        rdata[x + y*nx] = data[x + y*nx] * exp(-d2/sigma2);
      }
  }
  UNPROTECT(nprotect);
  return res;
"
funx <- cfunction(signature(a="array", s="numeric", centre="numeric"), code)

x <- array(runif(50*50), c(50,50,1))
res <- funx(a=x, s=10, centre=c(25,15))
if (interactive()) image(res[,,1])

## Same but done by registering an S4 method
setCMethod("funy", signature(a="array", s="numeric", centre="numeric"), code, verbose=TRUE)

res <- funy(x, 10, c(35,35))
if (interactive()) { x11(); image(res[,,1]) }

}

\author{
  Oleg Sklyar, Duncan Murdoch, Mike Smith, Dirk Eddelbuettel
}

\keyword{file}

\name{inline-package}
\alias{inline-package}
\alias{inline}
\docType{package}
\title{\packageTitle{inline}}
\description{\packageDescription{inline}}
\seealso{\code{\link{cfunction}}, \code{\link{cxxfunction}}}
\author{\packageAuthor{inline}}
\section{Maintainer}{\packageMaintainer{inline}}
\keyword{package}

=head1 NAME

B<Inline::Config> - Set configuration options for C<Inline>

=head1 SYNOPSIS

 BEGIN {
     use Inline;
     $Inline::Config::BUILD_PREFIX = '/tmp/Inline';
     $Inline::Config::INSTALL_PREFIX = '/home/ingy/Inline';
     $Inline::Config::FORCE_BUILD = 1;
     $Inline::Config::MAKEFILE{INC} = '-I/usr/include/other');
 }

or

 BEGIN {
     use Inline;
     Inline::Config::Build_Prefix('/tmp/Inline');
     Inline::Config::Install_Prefix('/home/ingy/Inline');
     Inline::Config::force_build(1);
     Inline::Config::makefile(INC => '-I/usr/include/other');
 }

or

 BEGIN {
     use Inline;
     my $ic = Inline::Config->new;
     $ic->build_prefix('/tmp/Inline');
     $ic->install_prefix('/home/ingy/Inline');
     $ic->force_build(1);
     $ic->makefile(INC => '-I/usr/include/other');
 }

or

 BEGIN {
     use Inline;
     Inline::Config->new
       ->build_prefix('/tmp/Inline')
       ->install_prefix('/home/ingy/Inline')
       ->force_build(1)
       ->makefile(INC => '-I/usr/include/other');
 }

=head1 DESCRIPTION

C<Inline::Config> is a helper module for C<Inline.pm>. It sets and saves all of C<Inline>'s configuration options and defaults. 

Currently there are two categories of settings: I<global> and I<makefile>. Global settings are a collection of various options. See L<"Global Settings"> below. Makefile settings are passed directly into the Makefile.PL file. See L<ExtUtils::MakeMaker> for an (long) list of available options.

=head2 Methods

All of the settings are stored as global variables in the C<Inline::Config> namespace and are spelled with capital letters. You can set them in several manners depending on your personal style (as shown in L<"SYNOPSIS"> above). 

 Direct:     $Inline::Config::BUILD_PREFIX = '/tmp/Inline';
 Function:   Inline::Config::Build_Prefix('/tmp/Inline');
 OO:         $ic->build_prefix('/tmp/Inline');
 Stacked OO: $ic->build_prefix('/tmp/Inline')->force_build(1);

When using the direct method, the name must be all capitals. When using the procedural methods, any case can be used.

=head2 Roll Your Own

Another option is to create your own C<Inline::Config.pm> module and set the base options there. Install the module higher up in the C<@::INC> path. Then you don't have to set the same options in all of your scripts. Use the C<PERL5LIB> environment variable, if you don't have install access to C<@::INC> on your system.

=head2 Global Settings

This is an alphabetical list of all the global settings currentlly available for C<Inline>. Many of them include command line shortcuts.

=over 4

=item

B<AUTO_INCLUDE_C> - Text to automatically prepend to your Inlined C<C> code. Normally used for header files that you always want. Default:

 #include "EXTERN.h"
 #include "perl.h"
 #include "XSUB.h"

=item

B<BUILD_PREFIX> - Tells what directory C<Inline> should install the compiled code under. This is a base directory. "C<make install>" actually creates a bunch of other directories under this. (Default is generated by examining your system for the best possible location. Program dies if none is found.)

=item

B<CLEAN_AFTER_BUILD> - Tells C<Inline> to remove the build files after compiling. You can also turn this option B<off> from the command line with the special syntax "C<perl -MInline=NOCLEAN ...>". (Default = 1)

=item

B<CLEAN_BUILD_AREA> - Tells C<Inline> to remove any build directories that may be lying around in your build area. Normally these directories get removed immediately after a successful build. Exceptions are when the build fails, or when you use the NOCLEAN or REPORTBUG options. You can also turn this option B<on> from the command line with the special syntax "C<perl -MInline=CLEAN ...>". (Default = 0)

=item

B<FORCE_BUILD> - Forces the code to be recompiled, even if everything is up to date. You can also turn this option on from the command line with the special syntax "C<perl -MInline=FORCE ...>". (Default = 0)

=item

B<PRINT_INFO> - This is a very useful option when you want to know what's going on under the hood. It tells C<Inline> to print helpful information to C<STDERR>. Among the things that get printed is a list of which C<Inline> functions were successfully bound to Perl. You can also set this option from the command line with the special syntax "C<perl -MInline=INFO ...>". (Default = 0)

=item

B<INSTALL_LIB> - Tells what directory C<Inline> should add to C<@::INC> to find your compiled code. This is based off of "C<INSTALL_PREFIX>", so you shouldn't need to set this directly.

B<NOTE> - The code to calculate the INSTALL_LIB directory path is tricky, because it depends on what version of perl you have. If you have problems with functions not being found, you may need to set this yourself. Hopefully this will all get figured out soon.

=item

B<INSTALL_PREFIX> - Tells what directory C<Inline> should install the compiled code under. This is a base directory. "C<make install>" actually creates a bunch of other directories under this. (Default is generated by examining your system for the best possible location. Program dies if none is found.)

=item

B<REPORTBUG> - Puts Inline into 'REPORTBUG' mode, which does special processing when you want to report a bug. REPORTBUG also automatically forces a build, and doesn't clean up afterwards. This is so that you can tar and mail the build directory to me. REPORTBUG will print exact instructions on what to do. You can also set this option from the command line with the special syntax "C<perl -MInline=REPORTBUG ...>". (Default = 0)

=item

B<SITE_INSTALL> - Says that compiled code should be installed in the Perl installation's "C<site_perl>" directory. Use this to permanently install an Inlined module. You can also set this option from the command line with the special syntax "C<perl -MInline=SITE_INSTALL ...>". (Default = 0)

=item

B<TEMP_DIR> - The temporary directory that "C<BUILD_PREFIX>" and "C<INSTALL_PREFIX>" are based off of. Setting this one, in effect, sets the other two. This directory must exist and be writable or C<Inline> will croak. 

If you do not set this, then C<Inline> will attempt to set a reasonable default for you. See L<Inline/"Find a Place to Build and Install"> for more info.

=back

=head1 AUTHOR

Brian Ingerson <INGY@cpan.org>

=head1 COPYRIGHT

Copyright (c) 2000, Brian Ingerson. All Rights Reserved.  This module is free software. It may be used, redistributed and/or modified under the terms of the Perl Artistic License
  (see http://www.perl.com/perl/misc/Artistic.html)

=cut

=pod

=for comment
DO NOT EDIT. This Pod was generated by Swim v0.1.38.
See http://github.com/ingydotnet/swim-pm#readme

=encoding utf8

=head1 NAME

Inline - Write Perl Subroutines in Other Programming Languages

=head1 VERSION

This document describes L<Inline> version B<0.79>.
";

=head1 SYNOPSIS

    use Inline C;

    print "9 + 16 = ", add(9, 16), "\n";
    print "9 - 16 = ", subtract(9, 16), "\n";

    __END__
    __C__
    int add(int x, int y) {
      return x + y;
    }

    int subtract(int x, int y) {
      return x - y;
    }

=head1 DESCRIPTION

The Inline module allows you to put source code from other programming
languages directly "inline" in a Perl script or module. The code is
automatically compiled as needed, and then loaded for immediate access
from Perl.

Inline saves you from the hassle of having to write and compile your own glue
code using facilities like XS or SWIG. Simply type the code where you want it
and run your Perl as normal. All the hairy details are handled for you. The
compilation and installation of your code chunks all happen transparently; all
you will notice is the delay of compilation on the first run.

The Inline code only gets compiled the first time you run it (or whenever it
is modified) so you only take the performance hit once. Code that is Inlined
into distributed modules (like on the CPAN) will get compiled when the module
is installed, so the end user will never notice the compilation time.

Best of all, it works the same on both Unix and Microsoft Windows. See L<Inline-
Support> for support information.

=head2 Why Inline?

Do you want to know "Why would I use other languages in Perl?" or "Why should
I use Inline to do it?"? I'll try to answer both.

=over

=item Why would I use other languages in Perl?

The most obvious reason is performance. For an interpreted language, Perl is
very fast. Many people will say "Anything Perl can do, C can do faster". (They
never mention the development time :-) Anyway, you may be able to remove a
bottleneck in your Perl code by using another language, without having to
write the entire program in that language. This keeps your overall development
time down, because you're using Perl for all of the non-critical code.

Another reason is to access functionality from existing API-s that use the
language. Some of this code may only be available in binary form. But by
creating small subroutines in the native language, you can "glue" existing
libraries to your Perl. As a user of the CPAN, you know that code reuse is a
good thing. So why throw away those Fortran libraries just yet?

If you are using Inline with the C language, then you can access the full
internals of Perl itself. This opens up the floodgates to both extreme power
and peril.

Maybe the best reason is "Because you want to!". Diversity keeps the world
interesting. TMTOWTDI!

=item Why should I use Inline to do it?

There are already two major facilities for extending Perl with C. They are XS
and SWIG. Both are similar in their capabilities, at least as far as Perl is
concerned. And both of them are quite difficult to learn compared to Inline.

There is a big fat learning curve involved with setting up and using the XS
environment. You need to get quite intimate with the following docs:

=over

=item * perlxs

=item * perlxstut

=item * perlapi

=item * perlguts

=item * perlmod

=item * h2xs

=item * xsubpp

=item * ExtUtils::MakeMaker

=back

With Inline you can be up and running in minutes. There is a C Cookbook with
lots of short but complete programs that you can extend to your real-life
problems. No need to learn about the complicated build process going on in the
background. You don't even need to compile the code yourself. Inline takes
care of every last detail except writing the C code.

Perl programmers cannot be bothered with silly things like compiling. "Tweak,
Run, Tweak, Run" is our way of life. Inline does all the dirty work for you.

Another advantage of Inline is that you can use it directly in a script. You
can even use it in a Perl one-liner. With XS and SWIG, you always set up an
entirely separate module. Even if you only have one or two functions. Inline
makes easy things easy, and hard things possible. Just like Perl.

Finally, Inline supports several programming languages (not just C and
C++). As of this writing, Inline has support for C, C++, Java, Python,
Ruby, Tcl, Assembler, Basic, Guile, Befunge, Octave, Awk, BC, TT (Template
Toolkit), WebChat and even PERL. New Inline Language Support Modules
(ILSMs) are regularly being added. See L<Inline-API> for details on how to
create your own ILSM.

=back

=head1 USING THE INLINE.PM MODULE

Inline is a little bit different than most of the Perl modules that you are
used to. It doesn't import any functions into your namespace and it doesn't
have any object oriented methods. Its entire interface (with two minor
exceptions) is specified through the C<'use Inline ...'> command.

This section will explain all of the different ways to C<use Inline>. If you
want to begin using C with Inline immediately, see L<Inline::C-Cookbook>.

=head2 The Basics

The most basic form for using Inline is:

    use Inline X => "X source code";

where 'X' is one of the supported Inline programming languages. The second
parameter identifies the source code that you want to bind to Perl. The source
code can be specified using any of the following syntaxes:

=over

=item The DATA Keyword.

    use Inline Java => 'DATA';

    # Perl code goes here ...

    __DATA__
    __Java__
    /* Java code goes here ... */

The easiest and most visually clean way to specify your source code in an
Inline Perl program is to use the special C<DATA> keyword. This tells Inline
to look for a special marker in your C<DATA> filehandle's input stream. In
this example the special marker is C<__Java__>, which is the programming
language surrounded by double underscores.

In case you've forgotten, the C<DATA> pseudo file is comprised of all the text
after the C<__END__> or C<__DATA__> section of your program. If you're working
outside the C<main> package, you'd best use the C<__DATA__> marker or else
Inline will not find your code.

Using this scheme keeps your Perl code at the top, and all the ugly Java stuff
down below where it belongs. This is visually clean and makes for more
maintainable code. An excellent side benefit is that you don't have to escape
any characters like you might in a Perl string. The source code is verbatim.
For these reasons, I prefer this method the most.

The only problem with this style is that since Perl can't read the C<DATA>
filehandle until runtime, it obviously can't bind your functions until
runtime. The net effect of this is that you can't use your Inline functions as
barewords (without predeclaring them) because Perl has no idea they exist
during compile time.

=item The FILE and BELOW keywords.

    use Inline::Files;
    use Inline Java => 'file';

    # Perl code goes here ...

    __JAVA__
    /* Java code goes here ... */

This is the newest method of specifying your source code. It makes use of the
Perl module C<Inline::Files> written by Damian Conway. The basic style and
meaning are the same as for the C<DATA> keyword, but there are a few syntactic
and semantic twists.

First, you must say 'use Inline::Files' before you 'use Inline' code that
needs those files. The special 'C<DATA>' keyword is replaced by either
'C<file>' or 'C<below>'. This allows for the bad pun idiom of:

    use Inline C => 'below';

You can omit the C<__DATA__> tag now. Inline::Files is a source filter that
will remove these sections from your program before Perl compiles it. They are
then available for Inline to make use of. And since this can all be done at
compile time, you don't have to worry about the caveats of the 'DATA' keyword.

This module has a couple small gotchas. Since Inline::Files only recognizes
file markers with capital letters, you must specify the capital form of
your language name. Also, there is a startup time penalty for using a
source code filter.

At this point Inline::Files is alpha software and use of it is experimental.
Inline's integration of this module is also fledgling at the time being. One
of things I plan to do with Inline::Files is to get line number info so when
an extension doesn't compile, the error messages will point to the correct
source file and line number.

My best advice is to use Inline::Files for testing (especially as support for
it improves), but use DATA for production and distributed/CPAN code.

=item Strings

    use Inline Java => <<'END';

    /* Java code goes here ... */
    END

    # Perl code goes here ...

You also just specify the source code as a single string. A handy way to write
the string is to use Perl's "here document" style of quoting. This is ok for
small functions but can get unwieldy in the large. On the other hand, the
string variant probably has the least startup penalty and all functions are
bound at compile time.

If you wish to put the string into a scalar variable, please be aware that the
C<use> statement is a compile time directive. As such, all the variables it
uses must also be set at compile time, C<before> the 'use Inline' statement.
Here is one way to do it:

    my $code;
    BEGIN {
        $code = <<END;

    /* Java code goes here ... */
    END
    }
    use Inline Java => $code;

    # Perl code goes here ...

=item The bind() Function

An alternative to using the BEGIN block method is to specify the source code
at run time using the 'Inline->bind()' method. (This is one of the interface
exceptions mentioned above) The C<bind()> method takes the same arguments as
C<'use Inline ...'>.

    my $code = <<END;

    /* Java code goes here ... */
    END

    Inline->bind(Java => $code);

You can think of C<bind()> as a way to C<eval()> code in other programming
languages.

Although bind() is a powerful feature, it is not recommended for use in Inline
based modules. In fact, it won't work at all for installable modules. See
instructions below for creating modules with Inline.

=item Other Methods

The source code for Inline can also be specified as an external filename, a
reference to a subroutine that returns source code, or a reference to an array
that contains lines of source code. (Note that if the external source file is
in the current directory it must be specified with a leading '.I<' - ie
'.>file.ext' instead of simply 'file.ext'.) These methods are less frequently
used but may be useful in some situations.

=item Shorthand

If you are using the 'DATA' or 'file' methods described above B<and> there are
no extra parameters, you can omit the keyword altogether. For example:

    use Inline 'Java';

    # Perl code goes here ...

    __DATA__
    __Java__
    /* Java code goes here ... */

or

    use Inline::Files;
    use Inline 'Java';

    # Perl code goes here ...

    __JAVA__
    /* Java code goes here ... */

=back

=head2 More about the DATA Section

If you are writing a module, you can also use the DATA section for POD and
AutoLoader subroutines. Just be sure to put them before the first Inline
marker. If you install the helper module C<Inline::Filters>, you can even
use POD inside your Inline code. You just have to specify a filter to
strip it out.

You can also specify multiple Inline sections, possibly in different
programming languages. Here is another example:

    # The module Foo.pm
    package Foo;
    use AutoLoader;

    use Inline C;
    use Inline C => DATA => filters => 'Strip_POD';
    use Inline Python;

    1;

    __DATA__

    sub marine {
        # This is an autoloaded subroutine
    }

    =head1 External subroutines

    =cut

    __C__
    /* First C section */

    __C__
    /* Second C section */
    =head1 My C Function

    Some POD doc.

    =cut

    __Python__
    """A Python Section"""

An important thing to remember is that you need to have one C<< use Inline Foo
=> 'DATA' >> for each C<__Foo__> marker, and they must be in the same order.
This allows you to apply different configuration options to each section.

=head2 Configuration Options

Inline tries to do the right thing as often as possible. But sometimes you may
need to override the default actions. This is easy to do. Simply list the
Inline configuration options after the regular Inline parameters. All
configuration options are specified as (key, value) pairs.

    use Inline (C => 'DATA',
                directory => './inline_dir',
                libs => '-lfoo',
                inc => '-I/foo/include',
                prefix => 'XXX_',
                warnings => 0,
               );

You can also specify the configuration options on a separate Inline call
like this:

    use Inline (C => Config =>
                directory => './inline_dir',
                libs => '-lfoo',
                inc => '-I/foo/include',
                prefix => 'XXX_',
                warnings => 0,
               );
    use Inline C => <<'END_OF_C_CODE';

The special keyword C<'Config'> tells Inline that this is a configuration-only
call. No source code will be compiled or bound to Perl.

If you want to specify global configuration options that don't apply to a
particular language, just leave the language out of the call. Like this:

    use Inline Config => warnings => 0;

The Config options are inherited and additive. You can use as many Config
calls as you want. And you can apply different options to different code
sections. When a source code section is passed in, Inline will apply
whichever options have been specified up to that point. Here is a complex
configuration example:

    use Inline (Config =>
                directory => './inline_dir',
               );
    use Inline (C => Config =>
                libs => '-lglobal',
               );
    use Inline (C => 'DATA',         # First C Section
                libs => ['-llocal1', '-llocal2'],
               );
    use Inline (Config =>
                warnings => 0,
               );
    use Inline (Python => 'DATA',    # First Python Section
                libs => '-lmypython1',
               );
    use Inline (C => 'DATA',         # Second C Section
                libs => [undef, '-llocal3'],
               );

The first C<Config> applies to all subsequent calls. The second C<Config>
applies to all subsequent C<C> sections (but not C<Python> sections). In the
first C<C> section, the external libraries C<global>, C<local1> and C<local2>
are used. (Most options allow either string or array ref forms, and do the
right thing.) The C<Python> section does not use the C<global> library, but
does use the same C<DIRECTORY>, and has warnings turned off. The second C<C>
section only uses the C<local3> library. That's because a value of C<undef>
resets the additive behavior.

The C<directory> and C<warnings> options are generic Inline options. All other
options are language specific. To find out what the C<C> options do, see
C<Inline::C>.

=head2 On and Off

If a particular config option has value options of 1 and 0, you can use the
'enable' and 'disable' modifiers. In other words, this:

    use Inline Config =>
               force_build => 1,
               clean_after_build => 0;

could be reworded as:

    use Inline Config =>
               enable => force_build =>
               disable => clean_after_build;

=head2 Playing 'with' Others

Inline has a special configuration syntax that tells it to get more
configuration options from other Perl modules. Here is an example:

    use Inline with => 'Event';

This tells Inline to load the module C<Event.pm> and ask it for configuration
information. Since C<Event> has a C API of its own, it can pass Inline all of
the information it needs to be able to use C<Event> C callbacks seamlessly.

That means that you don't need to specify the typemaps, shared libraries,
include files and other information required to get this to work.

You can specify a single module or a list of them. Like:

    use Inline with => qw(Event Foo Bar);

Currently, C<Event> is the only module that works I<with> Inline.

In order to make B<your> module work I<with> Inline in this way, your module
needs to provide a class method called C<Inline> that takes an Inline language
as a parameter (e.g. "C"), and returns a reference to a hash with
configuration information that is acceptable to the relevant ILSM. For C, see
L<C Configuration Options|Inline::C>. E.g.:

    my $confighashref = Event->Inline('C'); # only supports C in 1.21
    # hashref contains keys INC, TYPEMAPS, MYEXTLIB, AUTO_INCLUDE, BOOT

If your module uses L<ExtUtils::Depends> version 0.400 or higher, your module
only needs this:

    package Module;
    use autouse Module::Install::Files => qw(Inline);

=head2 Inline Shortcuts

Inline lets you set many configuration options from the command line. These
options are called 'shortcuts'. They can be very handy, especially when you
only want to set the options temporarily, for say, debugging.

For instance, to get some general information about your Inline code in the
script C<Foo.pl>, use the command:

    perl -MInline=info Foo.pl

If you want to force your code to compile, even if its already done, use:

    perl -MInline=force Foo.pl

If you want to do both, use:

    perl -MInline=info -MInline=force Foo.pl

or better yet:

    perl -MInline=info,force Foo.pl

=head2 The Inline 'directory'

Inline needs a place to build your code and to install the results of the
build. It uses a single directory named C<'.Inline/'> under normal
circumstances. If you create this directory in your home directory, the
current directory or in the directory where your program resides, Inline will
find and use it. You can also specify it in the environment variable
C<PERL_INLINE_DIRECTORY> or directly in your program, by using the
C<directory> keyword option. If Inline cannot find the directory in any of
these places it will create a C<'_Inline/'> directory in either your current
directory or the directory where your script resides.

One of the key factors to using Inline successfully, is understanding this
directory. When developing code it is usually best to create this directory
(or let Inline do it) in your current directory. Remember that there is
nothing sacred about this directory except that it holds your compiled code.
Feel free to delete it at any time. Inline will simply start from scratch and
recompile your code on the next run. If you have several programs that you
want to force to recompile, just delete your C<'.Inline/'> directory.

It is probably best to have a separate C<'.Inline/'> directory for each
project that you are working on. You may want to keep stable code in the
<.Inline/> in your home directory. On multi-user systems, each user should
have their own C<'.Inline/'> directories. It could be a security risk to put
the directory in a shared place like C</tmp/>.

=head2 Debugging Inline Errors

All programmers make mistakes. When you make a mistake with Inline, like
writing bad C code, you'll get a big error report on your screen. This report
tells you where to look to do the debugging. Some languages may also dump out
the error messages generated from the build.

When Inline needs to build something it creates a subdirectory under your
C<DIRECTORY/build/> directory. This is where it writes all the components it
needs to build your extension. Things like XS files, Makefiles and output
log files.

If everything goes OK, Inline will delete this subdirectory. If there is
an error, Inline will leave the directory intact and print its location.
The idea is that you are supposed to go into that directory and figure out
what happened.

Read the doc for your particular Inline Language Support Module for more
information.

=head2 The 'config' Registry File

Inline keeps a cached file of all of the Inline Language Support Module's meta
data in a file called C<config>. This file can be found in your C<directory>
directory. If the file does not exist, Inline creates a new one. It will
search your system for any module beginning with C<Inline::>. It will then
call that module's C<register()> method to get useful information for future
invocations.

Whenever you add a new ILSM, you should delete this file so that Inline will
auto-discover your newly installed language module. (This should no longer be
necessary as of Inline-0.49.)

=head1 CONFIGURATION OPTIONS

This section lists all of the generic Inline configuration options. For
language specific configuration, see the doc for that language.

=over

=item C<directory>

The C<directory> config option is the directory that Inline uses to both build
and install an extension.

Normally Inline will search in a bunch of known places for a directory called
C<'.Inline/'>. Failing that, it will create a directory called C<'_Inline/'>

If you want to specify your own directory, use this configuration option.

Note that you must create the C<directory> directory yourself. Inline will not
do it for you.

=item C<name>

You can use this option to set the name of your Inline extension object
module. For example:

    use Inline C => 'DATA',
               name => 'Foo::Bar';

would cause your C code to be compiled in to the object:

    lib/auto/Foo/Bar/Bar.so
    lib/auto/Foo/Bar/Bar.inl

(The .inl component contains dependency information to make sure the source
code is in sync with the executable)

If you don't use C<name>, Inline will pick a name for you based on your
program name or package name. In this case, Inline will also enable the
C<autoname> option which mangles in a small piece of the MD5 fingerprint into
your object name, to make it unique.

=item C<autoname>

This option is enabled whenever the C<name> parameter is not specified. To
disable it say:

    use Inline C => 'DATA',
               disable => 'autoname';

C<autoname> mangles in enough of the MD5 fingerprint to make your module name
unique. Objects created with C<autoname> will never get replaced. That also
means they will never get cleaned up automatically.

C<autoname> is very useful for small throw away scripts. For more serious
things, always use the C<name> option.

=item C<version>

Specifies the version number of the Inline extension object. It is used
B<only> for modules, and it must match the global variable $VERSION.
Additionally, this option should used if (and only if) a module is being set
up to be installed permanently into the Perl sitelib tree. Inline will croak
if you use it otherwise.

The presence of the C<version> parameter is the official way to let Inline
know that your code is an installable/installed module. Inline will never
generate an object in the temporary cache (C<_Inline/> directory) if
C<version> is set. It will also never try to recompile a module that was
installed into someone's Perl site tree.

So the basic rule is develop without C<version>, and deliver with C<version>.

=item C<with>

C<with> can also be used as a configuration option instead of using the
special 'with' syntax. Do this if you want to use different sections of Inline
code I<with> different modules. (Probably a very rare usage)

    use Event;
    use Inline C => DATA => with => 'Event';

Modules specified using the config form of C<with> will B<not> be
automatically required. You must C<use> them yourself.

=item C<using>

You can override modules that get used by ILSMs with the C<using> option. This
is typically used to override the default parser for L<Inline::C>, but might
be used by any ILSM for any purpose.

    use Inline config => using => '::Parser::RecDescent';
    use Inline C => '...';

This would tell L<Inline::C> to use L<Inline::C::Parser::RecDescent>.

=item C<global_load>

This option is for compiled languages only. It tells Inline to tell DynaLoader
to load an object file in such a way that its symbols can be dynamically
resolved by other object files. May not work on all platforms. See the
C<global> shortcut below.

=item C<untaint>

You can use this option whenever you use Perl's C<-T> switch, for taint
checking. This option tells Inline to blindly untaint all tainted variables.
(This is generally considered to be an appallingly insecure thing to do, and
not to be recommended - but the option is there for you to use if you want.
Please consider using something other than Inline for scripts that need taint
checking.) It also turns on C<safemode> by default. See the C<untaint>
shortcut below. You will see warnings about blindly untainting fields in both
%ENV and Inline objects. If you want to silence these warnings, set the Config
option C<no_untaint_warn> => 1. There can be some problems untainting Inline
scripts where older versions of Cwd, such as those that shipped with early
versions of perl-5.8 (and earlier), are installed. Updating Cwd will probably
solve these problems.

=item safemode

Perform extra safety checking, in an attempt to thwart malicious code. This
option cannot guarantee security, but it does turn on all the currently
implemented checks. (Currently, the only "currently implemented check" is to
ensure that the C<directory> option has also been used.)

There is a slight startup penalty by using C<safemode>. Also, using C<untaint>
automatically turns this option on. If you need your code to start faster
under C<-T> (taint) checking, you'll need to turn this option off manually.
Only do this if you are not worried about security risks. See the C<unsafe>
shortcut below.

=item C<force_build>

Makes Inline build (compile) the source code every time the program is run.
The default is 0. See the C<force> shortcut below.

=item C<build_noisy>

Tells ILSMs that they should dump build messages to the terminal rather than
be silent about all the build details.

=item C<build_timers>

Tells ILSMs to print timing information about how long each build phase took.
Usually requires C<Time::HiRes>.

=item C<clean_after_build>

Tells Inline to clean up the current build area if the build was successful.
Sometimes you want to C<disable> this for debugging. Default is 1. See the
C<noclean> shortcut below.

=item C<clean_build_area>

Tells Inline to clean up the old build areas within the entire Inline
C<directory>. Default is 0. See the C<clean> shortcut below.

=item C<print_info>

Tells Inline to print various information about the source code. Default is 0.
See the C<info> shortcut below.

=item C<print_version>

Tells Inline to print version info about itself. Default is 0. See the
C<version> shortcut below.

=item C<reportbug>

Puts Inline into 'reportbug' mode, which is what you want if you desire to
report a bug.

=item C<rewrite_config_file>

Default is 0, but setting C<< rewrite_config_file => 1 >> will mean that the
existing configuration file in the Inline C<directory> will be overwritten.
(This is useful if the existing config file is not up to date as regards
supported languages.)

=item C<warnings>

This option tells Inline whether to print certain warnings. Default is 1.

=back

=head1 INLINE CONFIGURATION SHORTCUTS

This is a list of all the shortcut configuration options currently available
for Inline. Specify them from the command line when running Inline scripts.

    perl -MInline=noclean inline_script.pl

or

    perl -MInline=info,force,noclean inline_script.pl

You can specify multiple shortcuts separated by commas. They are not case
sensitive. You can also specify shortcuts inside the Inline program like this:

    use Inline 'info', 'force', 'noclean';

NOTE: If a C<'use Inline'> statement is used to set shortcuts, it can not be
      used for additional purposes.

=over

=item C<clean>

Tells Inline to remove any build directories that may be lying around in your
build area. Normally these directories get removed immediately after a
successful build. Exceptions are when the build fails, or when you use the
C<noclean> or C<reportbug> options.

=item C<force>

Forces the code to be recompiled, even if everything is up to date.

=item C<global>

Turns on the C<global_load> option.

=item C<info>

This is a very useful option when you want to know what's going on under the
hood. It tells Inline to print helpful information to C<STDERR>. Among the
things that get printed is a list of which Inline functions were successfully
bound to Perl.

=item C<noclean>

Tells Inline to leave the build files after compiling.

=item C<noisy>

Use the C<build_noisy> option to print messages during a build.

=item C<reportbug>

Puts Inline into C<reportbug> mode, which does special processing when you
want to report a bug. C<reportbug> also automatically forces a build, and
doesn't clean up afterwards. This is so that you can tar and mail the build
directory to me. C<reportbug> will print exact instructions on what to do.
Please read and follow them carefully.

NOTE: C<reportbug> informs you to use the tar command. If your system does not
      have tar, please use the equivalent C<zip> command.

=item C<safe>

Turns C<safemode> on. C<untaint> will turn this on automatically. While this
mode performs extra security checking, it does not guarantee safety.

=item C<site_install>

This parameter used to be used for creating installable Inline modules. It has
been removed from Inline altogether and replaced with a much simpler and more
powerful mechanism, C<Inline::MakeMaker>. See the section below on how to
create modules with Inline.

=item C<_testing>

Used internally by CI<t>09parser.t and CI<t>10callback.t(in the Inline::C test
suite). Setting this option with Inline::C will mean that files named
C<parser_id> and C<void_test> are created in the C<./Inline_test> directory,
creating that directory if it doesn't already exist. The files (but not the
C<./Inline_test directory>) are cleaned up by calling
C<Inline::C::_testing_cleanup()>. Also used by C<t/06rewrite_config.t> to
trigger a warning.

=item C<timers>

Turn on C<build_timers> to get extra diagnostic info about builds.

=item C<unsafe>

Turns C<safemode> off. Use this in combination with C<untaint> for slightly
faster startup time under C<-T>. Only use this if you are sure the
environment is safe.

=item C<untaint>

Turn the C<untaint> option on. Used with C<-T> switch. In terms of secure
practices, this is definitely B<not> a recommended way of dealing with taint
checking, but it's the B<only> option currently available with Inline. Use it
at your own risk.

=item C<version>

Tells Inline to report its release version.

=back

=head1 WRITING MODULES WITH INLINE

Writing CPAN modules that use C code is easy with Inline. Let's say that you
wanted to write a module called C<Math::Simple>. Start by using the
following command:

    h2xs -PAXn Math::Simple

This will generate a bunch of files that form a skeleton of what you need for
a distributable module. (Read the h2xs manpage to find out what the options
do) Next, modify the C<Simple.pm> file to look like this:

    package Math::Simple;
    $VERSION = '1.23';

    use base 'Exporter';
    @EXPORT_OK = qw(add subtract);
    use strict;

    use Inline C => 'DATA',
               version => '1.23',
               name => 'Math::Simple';

    # The following Inline->init() call is optional - see below for more info.
    #Inline->init();

    1;

    __DATA__

    =pod

    =cut

    __C__
    int add(int x, int y) {
      return x + y;
    }

    int subtract(int x, int y) {
      return x - y;
    }

The important things to note here are that you B<must> specify a C<name> and
C<version> parameter. The C<name> must match your module's package name. The
C<version> parameter must match your module's C<$VERSION> variable and they
must be of the form C</^\d\.\d\d$/>.

NOTE: These are Inline's sanity checks to make sure you know what you're doing
      before uploading your code to CPAN. They insure that once the module has
      been installed on someone's system, the module would not get
      automatically recompiled for any reason. This makes Inline based modules
      work in exactly the same manner as XS based ones.

Finally, you need to modify the Makefile.PL. Simply change:

    use ExtUtils::MakeMaker;

to

    use Inline::MakeMaker;

And, in order that the module build work correctly in the cpan shell, add the
following directive to the Makefile.PL's WriteMakefile():

    CONFIGURE_REQUIRES  =>  {
        'Inline::MakeMaker'     => 0.45,
        'ExtUtils::MakeMaker'   => 6.52,
    },

This C<CONFIGURE_REQUIRES> directive ensures that the cpan shell will install
Inline on the user's machine (if it's not already present) before building
your Inline-based module. Specifying of "ExtUtils::MakeMaker => 6.52," is
optional, and can be omitted if you like. It ensures only that some harmless
warnings relating to the C<CONFIGURE_REQUIRES> directive won't be emitted
during the building of the module. It also means, of course, that
ExtUtils::Makemaker will first be updated on the user's machine unless the
user already has version 6.52 or later.

If the "Inline->init();" is not done then, having installed Math::Simple, a
warning that "One or more DATA sections were not processed by Inline" will
appear when (and only when) Math::Simple is loaded by a "require call. It's a
harmless warning - and if you're prepared to live with it, then there's no
need to make the "Inline->init();" call.

When the person installing C<Math::Simple> does a "C<make>", the generated
Makefile will invoke Inline in such a way that the C code will be compiled and
the executable code will be placed into the C<./blib> directory. Then when a
"C<make install>" is done, the module will be copied into the appropriate Perl
sitelib directory (which is where an installed module should go).

Now all you need to do is:

    perl Makefile.PL
    make dist

That will generate the file C<Math-Simple-0.20.tar.gz> which is a
distributable package. (It will also generate some harmless warnings in
relation to C<CONFIGURE_REQUIRES> unless the version of your
ExtUtils::MakeMaker is 6.52 or later.) That's all there is to it.

B<IMPORTANT NOTE>: Although the above steps will produce a workable module,
you still have a few more responsibilities as a budding new CPAN author. You
need to write lots of documentation and write lots of tests. Take a look at
some of the better CPAN modules for ideas on creating a killer test harness.
Actually, don't listen to me, go read these:

=over

=item * perldoc perlnewmod

=item * L<http://www.cpan.org/modules/04pause.html>

=item * L<http://www.cpan.org/modules/00modlist.long.html>

=back

=head1 HOW INLINE WORKS

In reality, Inline just automates everything you would need to do if you were
going to do it by hand (using XS, etc).

Inline performs the following steps:

=over

=item * Receive the Source Code

Inline gets the source code from your script or module with a statements like
the following:

    use Inline C => "Source-Code";

or

    use Inline;
    bind Inline C => "Source-Code";

where C<C> is the programming language of the source code, and C<Source-
Code> is a string, a file name, an array reference, or the special
C<'DATA'> keyword.

Since Inline is coded in a "C<use>" statement, everything is done during
Perl's compile time. If anything needs to be done that will affect the C<Source-
Code>, it needs to be done in a C<BEGIN> block that is I<before> the "C<use
Inline ...>" statement. If you really need to specify code to Inline at
runtime, you can use the C<bind()> method.

Source code that is stowed in the C<'DATA'> section of your code, is read in
by an C<INIT> subroutine in Inline. That's because the C<DATA> filehandle is
not available at compile time.

=item * Check if the Source Code has been Built

Inline only needs to build the source code if it has not yet been built. It
accomplishes this seemingly magical task in an extremely simple and
straightforward manner. It runs the source text through the C<Digest::MD5>
module to produce a 128-bit "fingerprint" which is virtually unique. The
fingerprint along with a bunch of other contingency information is stored in a
C<.inl> file that sits next to your executable object. For instance, the C<C>
code from a script called C<example.pl> might create these files:

    example_pl_3a9a.so
    example_pl_3a9a.inl

If all the contingency information matches the values stored in the C<.inl>
file, then proceed to step 8. (No compilation is necessary)

=item * Find a Place to Build and Install

At this point Inline knows it needs to build the source code. The first thing
to figure out is where to create the great big mess associated with
compilation, and where to put the object when it's done.

By default Inline will try to build and install under the first place that
meets one of the following conditions:

=over

=item 1.

The DIRECTORY= config option; if specified

=item 2.

The C<PERL_INLINE_DIRECTORY> environment variable; if set

=item 3.

C<.Inline/> (in current directory); if exists and C<$PWD != $HOME>

=item 4.

binI<.Inline> (in directory of your script); if exists

=item 5.

C<~/.Inline/> - if exists

=item 6.

C<./_Inline/> - if exists

=item 7.

C<bin/_Inline> - if exists

=item 8.

Create C<./_Inline/> - if possible

=item 9.

Create C<bin/_Inline/> - if possible

=back

Failing that, Inline will croak. This is rare and easily remedied by just
making a directory that Inline will use.

If the C<PERL_INSTALL_ROOT> Environment Variable has been set, you will need
to make special provision for that if the 'make install' phase of your Inline
scripts are to succeed.

If the module option is being compiled for permanent installation, then Inline
will only use C<./_Inline/> to build in, and the C<$Config{installsitearch}>
directory to install the executable in. This action is caused by
Inline::MakeMaker, and is intended to be used in modules that are to be
distributed on the CPAN, so that they get installed in the proper place.

=item * Parse the Source for Semantic Cues

Inline::C uses the module C<Parse::RecDescent> to parse through your chunks of
C source code and look for things that it can create run-time bindings to. In
C<C> it looks for all of the function definitions and breaks them down into
names and data types. These elements are used to correctly bind the C<C>
function to a C<Perl> subroutine. Other Inline languages like Python and Java
actually use the C<python> and C<javac> modules to parse the Inline code.

=item * Create the Build Environment

Now Inline can take all of the gathered information and create an environment
to build your source code into an executable. Without going into all the
details, it just creates the appropriate directories, creates the appropriate
source files including an XS file (for C) and a C<Makefile.PL>.

=item * Build the Code and Install the Executable

The planets are in alignment. Now for the easy part. Inline just does what you
would do to install a module. "`perl Makefile.PL && make && make test && make
install>". If something goes awry, Inline will croak with a message indicating
where to look for more info.

=item * Tidy Up

By default, Inline will remove all of the mess created by the build process,
assuming that everything worked. If the build fails, Inline will leave
everything intact, so that you can debug your errors. Setting the C<noclean>
shortcut option will also stop Inline from cleaning up.

=item * DynaLoad the Executable

For C (and C++), Inline uses the C<DynaLoader::bootstrap> method to pull your
external module into C<Perl> space. Now you can call all of your external
functions like Perl subroutines.

Other languages like Python and Java, provide their own loaders.

=back

=head1 SEE ALSO

For information about using Inline with C see L<Inline::C>.

For sample programs using Inline with C see L<Inline::C-Cookbook>.

For "Formerly Answered Questions" about Inline, see L<Inline-FAQ>.

For information on supported languages and platforms see L<Inline-Support>.

For information on writing your own Inline Language Support Module, see
L<Inline-API>.

Inline's mailing list is inline@perl.org

To subscribe, send email to inline-subscribe@perl.org

=head1 BUGS AND DEFICIENCIES

When reporting a bug, please do the following:

=over

=item * Put "use Inline 'reportbug';" at the top of your code, or use the command line option "perl -MInline=reportbug ...".

=item * Run your code.

=item * Follow the printed directions.

=back

=head1 AUTHOR

Ingy döt Net <ingy@cpan.org>

Sisyphus <sisyphus@cpan.org> fixed some bugs and is current co-maintainer.

=head1 COPYRIGHT

=over

=item * Copyright 2000-2015. Ingy döt Net.

=item * Copyright 2008, 2010-2014. Sisyphus.

=back

This program is free software; you can redistribute it and/or modify it under
the same terms as Perl itself.

See L<http://www.perl.com/perl/misc/Artistic.html>

=cut

=head1 NAME

Inline-API - How to bind a programming language to Perl using Inline.pm

=head1 SYNOPSIS

    #!/usr/bin/perl
    
    use Inline Foo;
    say_it('foo');  # Use Foo to print "Hello, Foo"
    
    __Foo__ 
    foo-sub say_it {
        foo-my $foo = foo-shift;
        foo-print "Hello, $foo\n";
    }

=head1 DESCRIPTION

So you think Inline C is pretty cool, but what you really need is for
Perl to work with the brand new programming language "Foo". Well you're
in luck. C<Inline.pm> has support for adding your own Inline Language
Support Module (B<ILSM>), like C<Inline::Foo>.

Inline has always been intended to work with lots of different
programming languages. Many of the details can be shared between
implementations, so that C<Inline::Java> has a similar interface to
C<Inline::ASM>. All of the common code is in C<Inline.pm>.

Language specific modules like C<Inline::Python> are subclasses of
C<Inline.pm>. They can inherit as much of the common behaviour as they
want, and provide specific behaviour of their own. This usually comes in
the form of Configuration Options and language specific compilation.

The Inline C support is probably the best boilerplate to copy from.
Since version 0.30 all C support was isolated into the module
C<Inline::C> and the parsing grammar is further broken out into
C<Inline::C::grammar>. All of these components come with the Inline
distribution.

This POD gives you all the details you need for implementing an ILSM.
For further assistance, contact inline@perl.org See L<"SEE ALSO"> below.

We'll examine the joke language Inline::Foo which is distributed with
Inline. It actually is a full functioning ILSM. I use it in Inline's
test harness to test base Inline functionality. It is very short, and
can help you get your head wrapped around the Inline API.

=head1 A Skeleton

For the remainder of this tutorial, let's assume we're writing an ILSM
for the ficticious language C<Foo>. We'll call it C<Inline::Foo>. Here
is the entire (working) implementation.

    package Inline::Foo;
    use strict;
    $Inline::Foo::VERSION = '0.01';
    @Inline::Foo::ISA = qw(Inline);
    require Inline;
    use Carp;
    
    #===========================================================
    # Register Foo as an Inline Language Support Module (ILSM)
    #===========================================================
    sub register {
        return {
            language => 'Foo',
            aliases => ['foo'],
            type => 'interpreted',
            suffix => 'foo',
           };
    }
    
    #===========================================================
    # Error messages
    #===========================================================
    sub usage_config { 
        my ($key) = @_;
        "'$key' is not a valid config option for Inline::Foo\n";
    }
    
    sub usage_config_bar { 
        "Invalid value for Inline::Foo config option BAR";
    }
    
    #===========================================================
    # Validate the Foo Config Options
    #===========================================================
    sub validate {
        my $o = shift;
        $o->{ILSM}{PATTERN} ||= 'foo-';
        $o->{ILSM}{BAR} ||= 0;
        while (@_) {
        my ($key, $value) = splice @_, 0, 2;
        if ($key eq 'PATTERN') {
            $o->{ILSM}{PATTERN} = $value;
            next;
        }
        if ($key eq 'BAR') {
            croak usage_config_bar
              unless $value =~ /^[01]$/;
            $o->{ILSM}{BAR} = $value;
            next;
        }
        croak usage_config($key);
        }
    }
    
    #===========================================================
    # Parse and compile Foo code
    #===========================================================
    sub build {
        my $o = shift;
        my $code = $o->{API}{code};
        my $pattern = $o->{ILSM}{PATTERN};
        $code =~ s/$pattern//g;
        $code =~ s/bar-//g if $o->{ILSM}{BAR};
        sleep 1;             # imitate compile delay
        {
            package Foo::Tester;
            eval $code;
        }
        croak "Foo build failed:\n$@" if $@;
        my $path = "$o->{API}{install_lib}/auto/$o->{API}{modpname}";
        my $obj = $o->{API}{location};
        $o->mkpath($path) unless -d $path;
        open FOO_OBJ, "> $obj"
          or croak "Can't open $obj for output\n$!";
        print FOO_OBJ $code;
        close \*FOO_OBJ;
    }
    
    #===========================================================
    # Only needed for interpreted languages
    #===========================================================
    sub load {
        my $o = shift;
        my $obj = $o->{API}{location};
        open FOO_OBJ, "< $obj"
          or croak "Can't open $obj for output\n$!";
        my $code = join '', <FOO_OBJ>;
        close \*FOO_OBJ;
        eval "package $o->{API}{pkg};\n$code";
        croak "Unable to load Foo module $obj:\n$@" if $@;
    }
    
    #===========================================================
    # Return a small report about the Foo code.
    #===========================================================
    sub info {
        my $o = shift;
        my $text = <<'END';
    This is a small report about the Foo code. Perhaps it contains
    information about the functions the parser found which will be
    bound to Perl. It will get included in the text produced by the
    Inline 'INFO' command.
    END
        return $text;
    }
    
    1;

Except for C<load()>, the subroutines in this code are mandatory for an
ILSM. What they do is described below. A few things to note:

=over 4

=item 1

C<Inline::Foo> must be a subclass of Inline. This is accomplished with:

    @Inline::Foo::ISA = qw(Inline);

=item 2

The line 'C<require Inline;>' is not necessary. But it is there to
remind you not to say 'C<use Inline;>'. This will not work.

=item 3

Remember, it is not valid for a user to say:

    use Inline::Foo;

C<Inline.pm> will detect such usage for you in its C<import> method,
which is automatically inherited since C<Inline::Foo> is a subclass.

=item 4

In the build function, you normally need to parse your source code.
Inline::C uses Parse::RecDescent to do this. Inline::Foo simply uses
eval. (After we strip out all occurances of 'foo-').

An alternative parsing method that works well for many ILSMs (like Java
and Python) is to use the language's compiler itself to parse for you.
This works as long as the compiler can be made to give back parse
information.

=back

=head1 The Inline API

This section is a more formal specification of what functionality you'll
need to provide to implement an ILSM.

When Inline determines that some C<Foo> code needs to be compiled it
will automatically load your ILSM module. It will then call various
subroutines which you need to supply. We'll call these subroutines
"callbacks".

You will need to provide the following 5 callback subroutines. 

=head2 The register() Callback

This subroutine receives no arguments. It returns a reference to a hash
of ILSM meta-data. Inline calls this routine only when it is trying to
detect new ILSM-s that have been installed on a given system. Here is an
example of the has ref you would return for Foo:

    {
     language => 'Foo',
     aliases => ['foo'],
     type => 'interpreted',
     suffix => 'foo',
    };

The meta-data items have the following meanings:

=over 4

=item language

This is the proper name of the language. It is usually implemented as
C<Inline::X> for a given language 'X'.

=item aliases

This is a reference to an array of language name aliases. The proper
name of a language can only contain word characters. [A-Za-z0-9_] An
alias can contain any characters except whitespace and quotes. This is
useful for names like 'C++' and 'C#'.

=item type

Must be set to 'compiled' or 'interpreted'. Indicates the category of
the language.

=item suffix

This is the file extension for the cached object that will be created.
For 'compiled' languages, it will probably be 'so' or 'dll'. The
appropriate value is in C<Config.pm>.

For interpreted languages, this value can be whatever you want. Python
uses C<pydat>. Foo uses C<foo>.

=back

=head2 The validate() Callback

This routine gets passed all configuration options that were not already
handled by the base Inline module. The options are passed as key/value
pairs. It is up to you to validate each option and store its value in
the Inline object (which is also passed in). If a particular option is
invalid, you should croak with an appropriate error message.

=head2 The build() Callback

This subroutine is responsible for doing the parsing and compilation of
the Foo source code. The Inline object is passed as the only argument.
All pertinent information will be stored in this object. C<build()> is
required to create a cache object of a specific name, or to croak with
an appropriate error message.

This is the meat of your ILSM. Since it will most likely be quite
complicated, it is probably best that you study an existing ILSM like
C<Inline::C>.

=head2 The load() Callback

This method only needs to be provided for interpreted languages. It's
responsibility is to start the interpreter.

For compiled languages, the load routine from C<Inline.pm> is called
which uses C<DynaLoader> to load the shared object or DLL.

=head2 The info() Callback

This method is called when the user makes use of the C<INFO>
shortcut. You should return a string containing a small report about
the Inlined code.

=head1 The Inline Object

C<Inline.pm> creates a hash based Perl object for each section of
Inlined source code it receives. This object contains lots of
information about the code, the environment, and the configuration
options used.

This object is a hash that is broken into several subhashes. The only
two subhashes that an ILSM should use at all are $o->{API} and
$o->{ILSM}. The first one contains all of the information that Inline
has gather for you in order for you to create/load a cached object of
your design. The second one is a repository where your ILSM can freely
store data that it might need later on.

This section will describe all of the Inline object "API" attributes.

=head2 The code Attribute

This the actual source code passed in by the user. It is stored as one
long string.

=head2 The language Attribute

The proper name of the language being used.

=head2 The language_id Attribute

The language name specified by the user. Could be 'C++' instead of 'CPP'.

=head2 The module Attribute

This is the shared object's file name.

=head2 The modfname Attribute

This is the shared object's file name.

=head2 The modpname Attribute

This is the shared object's installation path extension.

=head2 The version Attribute

The version of C<Inline.pm> being used.

=head2 The pkg Attribute

The Perl package from which this invocation pf Inline was called.

=head2 The install_lib Attribute

This is the directory to write the shared object into.

=head2 The build_dir Attribute

This is the directory under which you should write all of your build
related files.

=head2 The script Attribute

This is the name of the script that invoked Inline.

=head2 The location Attribute

This is the full path name of the executable object in question.

=head2 The suffix Attribute

This is the shared library extension name. (Usually 'so' or 'dll').

=over 4

=back

=head1 The Inline Namespace

C<Inline.pm> has been set up so that anyone can write their own language
support modules. It further allows anyone to write a different
implementation of an existing Inline language, like C for instance. You
can distribute that module on the CPAN.

If you have plans to implement and distribute an Inline module, I would
ask that you please work with the Inline community. We can be reached at
the Inline mailing list: inline@perl.org (Send mail to
inline-subscribe@perl.org to subscribe). Here you should find the advice
and assistance needed to make your module a success.

The Inline community will decide if your implementation of COBOL will be
distributed as the official C<Inline::COBOL> or should use an alternate
namespace. In matters of dispute, I (Brian Ingerson) retain final
authority. (and I hope not to need use of it :-) Actually
modules@perl.org retains the B<final> authority.

But even if you want to work alone, you are free and welcome to write
and distribute Inline language support modules on CPAN. You'll just need
to distribute them under a different package name.

=head1 SEE ALSO

For generic information about Inline, see L<Inline>.

For information about using Inline with C see L<Inline::C>.

For information on supported languages and platforms see
L<Inline-Support>.

Inline's mailing list is inline@perl.org

To subscribe, send email to inline-subscribe@perl.org

=head1 AUTHOR

Brian Ingerson <INGY@cpan.org>

=head1 COPYRIGHT

Copyright (c) 2000, 2001, 2002. Brian Ingerson. All rights reserved.

This program is free software; you can redistribute it and/or modify it
under the same terms as Perl itself.

See http://www.perl.com/perl/misc/Artistic.html

=cut

\name{internals}
\alias{internals}

\alias{setCMethod,character,list,list-method}
\alias{setCMethod,character,character,character-method}

\title{ Internals }

\description{
   Aliases required for 'R CMD check' but those noone will ever search for.
   Here to prevent the mess of the index. The corresponding items have help
   aliases without method signature!
}

\keyword{internal}

=pod

=for comment
DO NOT EDIT. This Pod was generated by Swim v0.1.38.
See http://github.com/ingydotnet/swim-pm#readme

=encoding utf8

=head1 NAME

Inline-Support - Support Information for Inline.pm and related modules.

=head1 DESCRIPTION

This document contains all of the latest support information for C<Inline.pm>
and the recognized Inline Language Support Modules (ILSMs) available on CPAN.

=head1 SUPPORTED LANGUAGES

The most important language that Inline supports is C<C>. That is because Perl
itself is written in C<C>. By giving a your Perl scripts access to C<C>, you
in effect give them access to the entire glorious internals of Perl. (Caveat
scriptor :-)

As of this writing, Inline also supports:

=over

=item * C++

=item * Java

=item * Python

=item * Tcl

=item * Assembly

=item * CPR

=item * And even Inline::Foo! :)

=back

Projects that I would most like to see happen in the year 2001 are:

=over

=item * Fortran

=item * Ruby

=item * Lisp

=item * Guile

=item * Bash

=item * Perl4

=back

=head1 SUPPORTED PLATFORMS

C<Inline::C> should work anywhere that CPAN extension modules (those that use
XS) can be installed, using the typical install format of:

    perl Makefile.PL
    make
    make test
    make install

It has been tested on many Unix and Windows variants.

B<NOTE>: C<Inline::C> requires Perl 5.005 or higher because
C<Parse::RecDescent> requires it. (Something to do with the C<qr> operator)

Inline has been successfully tested at one time or another on the following
platforms:

=over

=item * Linux

=item * Solaris

=item * SunOS

=item * HPUX

=item * AIX

=item * FreeBSD

=item * OpenBSD

=item * BeOS

=item * OS X

=item * WinNT

=item * Win2K

=item * WinME

=item * Win98

=item * Cygwin

=back

The Microsoft tests deserve a little more explanation. I used the following:

=over

=item * Windows NT 4.0 (service pack 6)

=item * Perl 5.005_03 (ActiveState build 522)

=item * MS Visual C++ 6.0

=item * The "nmake" make utility (distributed w/ Visual C++)

=back

C<Inline::C> pulls all of its base configuration (including which C<make>
utility to use) from C<Config.pm>. Since your MSWin32 version of Perl probably
came from ActiveState (as a binary distribution) the C<Config.pm> will
indicate that C<nmake> is the system's C<make> utility. That is because
ActiveState uses Visual C++ to compile Perl.

To install C<Inline.pm> (or any other CPAN module) on MSWin32 w/ Visual C++,
use these:

    perl Makefile.PL
    nmake
    nmake test
    nmake install

Inline has also been made to work with Mingw32/gcc on all Windows platforms.
This is a free compiler for Windows. You must also use a perl built with
that compiler.

The "Cygwin" test was done on a Windows 98 machine using the Cygwin Unix/Win32
porting layer software from Cygnus. The C<perl> binary on this machine was
also compiled using the Cygwin tool set (C<gcc>). This software is freely
available from L<http://sources.redhat.com/cygwin/>

If you get Inline to work on a new platform, please send me email email. If it
doesn't work, let me know as well and I'll see what can be done.

=head1 SEE ALSO

For general information about Inline see L<Inline>.

For information about using Inline with C see L<Inline::C>.

For sample programs using Inline with C see L<Inline::C-Cookbook>.

For information on writing your own Inline Language Support Module, see
L<Inline-API>.

Inline's mailing list is inline@perl.org

To subscribe, send email to inline-subscribe@perl.org

=head1 AUTHOR

ingy döt Net <ingy@cpan.org>

=head1 COPYRIGHT

Copyright 2000-2015. Ingy döt Net.

Copyright 2008, 2010, 2011. Sisyphus.

This program is free software; you can redistribute it and/or modify it under
the same terms as Perl itself.

See L<http://www.perl.com/perl/misc/Artistic.html>

=cut

\name{cxxfunction}
\alias{cxxfunction}
\alias{rcpp}
\title{inline C++ function}
\description{
Functionality to dynamically define an R function with inlined C++ code
using the \code{\link{.Call}} calling convention.

The \code{rcpp()} wrapper sets the plugin to the \dQuote{Rcpp} value
suitable for using \pkg{Rcpp}.
}
\usage{
cxxfunction(sig = character(), body = character(), 
            plugin = "default", includes = "", 
            settings = getPlugin(plugin), ..., verbose = FALSE)
rcpp(..., plugin="Rcpp")
}
\arguments{
  \item{sig}{Signature of the function. A named character vector}
  \item{body}{A character vector with C++ code to include in the body of the compiled C++ function}
  \item{plugin}{Name of the plugin to use. See \code{\link{getPlugin}} for details about plugins. }
  \item{includes}{User includes, inserted after the includes provided by the plugin. }
  \item{settings}{Result of the call to the plugin}
  \item{\dots}{Further arguments to the plugin}
  \item{verbose}{verbose output}
}
\value{A function}
\seealso{\code{\link{cfunction}}}
\examples{
\dontrun{
# default plugin
fx <- cxxfunction(signature(x = "integer", y = "numeric"), 
	          "return Rf_ScalarReal(INTEGER(x)[0] * REAL(y)[0]);")
fx(2L, 5)

# Rcpp plugin
if (requireNamespace("Rcpp", quietly=TRUE)) {

    fx <- cxxfunction(signature(x = "integer", y = "numeric"), 
                      "return wrap( as<int>(x) * as<double>(y));",
                      plugin = "Rcpp" )
    fx(2L, 5)

    ## equivalent shorter form using rcpp()
    fx <- rcpp(signature(x = "integer", y = "numeric"),
               "return wrap(as<int>(x) * as<double>(y));")
}

# RcppArmadillo plugin
if (requireNamespace(RcppArmadillo)) {
	
    fx <- cxxfunction(signature(x = "integer", y = "numeric"),
                      "int dim = as<int>(x);
		       arma::mat z = as<double>(y) * arma::eye<arma::mat>(dim, dim);
		       return wrap(arma::accu(z));",
                      plugin = "RcppArmadillo")
    fx(2L, 5)
}
}
}
\keyword{programming}
\keyword{interface}

=pod

=for comment
DO NOT EDIT. This Pod was generated by Swim v0.1.38.
See http://github.com/ingydotnet/swim-pm#readme

=encoding utf8

=head1 NAME

Inline-FAQ - The Inline FAQ

=head1 DESCRIPTION

Welcome to the official Inline FAQ. In this case, B<FAQ> means: B<Formerly
Answered Questions>

This is a collection of old, long-winded emails that myself and others have
sent to the Inline mailing list. (inline@perl.org) They have been reviewed and
edited for general Inline edification. Some of them may be related to a
specific language. They are presented here in a traditional FAQ layout.

=head1 GENERAL INLINE

Since there is only a handful of content so far, all FAQs are currently under
this heading.

=head2 How disposable is a C<.Inline> or C<_Inline> directory?

I probably need to be more emphatic about the role of C<_Inline/> cache
directories. Since they are created automatically, they are completely
disposable. I delete them all the time. And it is fine to have a different one
for each project. In fact as long as you don't have C<~/.Inline/> defined,
Inline will create a new C<./_Inline> directory (unless, you've done something
to override this automatic process - such as using the DIRECTORY config
option, or using the C<PERL_INLINE_DIRECTORY> environment variable). You can
move that to C<./.Inline> and it will continue to work if you want togive it
more longevity and hide it from view. There is a long complicated list of
rules about how C<[_.]Inline/> directories are used/created. But it was
designed to give you the most flexibility/ease-of-use. Never be afraid to nuke
'em. They'll just pop right back next time they're needed. :)

=head2 Whatever happened to the C<SITE_INSTALL> option?

C<SITE_INSTALL> is gone. I was going to leave it in and change the semantics,
but thought it better to remove it, so people wouldn't try to use it the old
way. There is now C<_INSTALL_> (but you're not supposed to know that :). It
works magically through the use of Inline::MakeMaker. I explained this earlier
but it's worth going through again because it's the biggest change for 0.40.
Here's how to 'permanently' install an Inline extension (Inline based module)
with 0.40:

=over

=item 1.

Create a module with Inline.

=item 2.

Test it using the normal / local C<_Inline/> cache.

=item 3.

Create a Makefile.PL (like the one produced by h2xs)

=item 4.

Change 'use ExtUtils::MakeMaker' to 'use Inline::MakeMaker'

=item 5.

In the Makefile.PL's WriteMakefile() insert:

    CONFIGURE_REQUIRES  =>  {
        'Inline::MakeMaker'     => 0.45,
        'ExtUtils::MakeMaker'   => 6.52,
    },

(See the "Writing Modules with Inline" section of Inline.pod for an
explanation / elaboration.)

=item 6.

Change your 'use Inline C => DATA' to 'use Inline C => DATA => NAME => Foo

=back

=> VERSION => 1.23' + Make sure NAME matches your package name ('Foo'), or
=> begins with 'Foo::'. + If you want to quiet a harmless warning that will
=> appear when the module is loaded via "require", do "Inline->init();". See
=> "Writing Modules with Inline" in the Inline pod for details. + Make sure
=> VERSION matches $Foo::VERSION. This must be a string (not a number)
=> matching C</^\d\.\d\d$/> + Do the perl / make / test / install dance
=> (thanks binkley :)

With Inline 0.41 (or thereabouts) you can skip steps 3 & 4, and just say
C<perl -MInline=INSTALL ./Foo.pm>. This will work for non-Inline modules too.
It will become the defacto standard (since there is no easy standard) way of
installing a Perl module. It will allow Makefile.PL parameters C<perl -
MInline=INSTALL ./Foo.pm - PREFIX=/home/ingy/perl> and things like that. It
will also make use of a MANIFEST if you provide one.

=head2 How do I create a binary distribution using Inline?

I've figured out how to create and install a PPM binary distribution;
with or without distributing the C code! And I've decided to share it
with all of you :)

NOTE: Future versions of Inline will make this process a one line command. But
      for now just use this simple recipe.

The Inline 0.40 distribution comes with a sample extension module called
Math::Simple. Theoretically you could distribute this module on CPAN. It has
all the necessary support for installation. You can find it in C<Inline-
0.40/modules/Math/Simple/>. Here are the steps for converting this into a
binary distribution B<without> C source code.

NOTE: The recipient of this binary distribution will need to have the
      PPM.pm module installed. This module requires a lot of other CPAN
      modules. ActivePerl (available for Win32, Linux, and Solaris) has all
      of these bundled. While ActivePerl isn't required, it makes things (a
      lot) easier.

=over

=item 1.

cd C<Inline-0.40/Math/Simple/>

=item 2.

Divide Simple.pm into two files:

    ---8<--- (Simple.pm)
    package Math::Simple;
    use strict;
    require Exporter;
    @Math::Simple::ISA = qw(Exporter);
    @Math::Simple::EXPORT = qw(add subtract);
    $Math::Simple::VERSION = '1.23';

    use Inline (C => 'src/Simple.c' =>
                NAME => 'Math::Simple',
                VERSION => '1.23',
               );
    1;
    ---8<---
    ---8<--- (src/Simple.c)
    int add (int x, int y) {
        return x + y;
    }

    int subtract (int x, int y) {
        return x - y;
    }
    ---8<---

=item 3.

now you have the Perl in one file and the C in the other. The C code must be

=back

in a subdirectory. + Note that I also changed the term 'DATA' to the name of
the C file. This will work just as if the C were still inline. + Run 'perl
Makefile.PL' + Run 'make test' + Get the MD5 key from
C<blib/arch/auto/Math/Simple/Simple.inl> + Edit C<blib/lib/Math/Simple.pm>.
Change C<src/Simple.c> to C<02c61710cab5b659efc343a9a830aa73> (the MD5 key)

=over

=item 1.

Run 'make ppd'

=item 2.

Edit 'Math-Simple.ppd'. Fill in AUTHOR and ABSTRACT if you wish. Then

=back

change:

      <CODEBASE HREF="" />

    to

      <CODEBASE HREF="Math-Simple.tar.gz" />

=over

=item 1.

Run:

    tar cvf Math-Simple.tar blib
    gzip --best Math-Simple.tar

=item 2.

Run:

    tar cvf Math-Simple-1.23.tar Math-Simple.ppd Math-Simple.tar.gz
    gzip --best Math-Simple-1.23.tar

=item 3.

Distribute Math-Simple-1.23.tar.gz with the following instructions:

=over

=item 1.

Run:

gzip -d Math-Simple-1.23.tar.gz tar xvzf Math-Simple-1.23.tar

=item 2.

Run 'ppm install Math-Simple.ppd'

=item 3.

Delete Math-Simple.tar and Math-Simple.ppd.

=item 4.

Test with:

perl -MMath::Simple -le 'print add(37, 42)'

=back

=back

That's it. The process should also work with zip instead of tar, but I
haven't tried it.

The recipient of the binary must have Perl built with a matching architecture.
Luckily, ppm will catch this.

For a binary dist B<with> C source code, simply omit steps 2, 3, 6, and 7.

If this seems too hard, then in a future version you should be able to
just type:

    make ppm

=head2 Why does C<C/t/09parser.t> fail on Cygwin ?

It doesn't always fail on Cygwin, but if you find that it produces "unable to
remap .... to same address as parent" errors during the build phase, then it's
time for you to run rebaseall.

See L<http://cygwin.com/faq/faq-nochunks.html#faq.using.fixing-fork-failures>
and, if needed, seek further help from the Cygwin mailing list.

=cut

\name{package.skeleton-methods}
\docType{methods}
\alias{package.skeleton-methods}
\alias{package.skeleton,ANY,ANY-method}
\alias{package.skeleton,character,CFunc-method}
\alias{package.skeleton,character,CFuncList-method}
\title{Generate the skeleton of a package}
\description{
Generate the skeleton of a package
}
\section{Methods}{
\describe{

\item{\code{signature(name = "ANY", list = "ANY")}}{
	Standard method. See \code{\link[utils]{package.skeleton}}
}

\item{\code{signature(name = "character", list = "CFunc")}}{
	Method for a single generated by \code{\link{cfunction}}
	or \code{\link{cxxfunction}}
}

\item{\code{signature(name = "character", list = "CFuncList")}}{
	Method for a set functions generated by \code{\link{cfunction}}
	or \code{\link{cxxfunction}}
}
}}
\examples{
\dontrun{

fx <- cxxfunction( signature(x = "integer", y = "numeric" ) , '
	return ScalarReal( INTEGER(x)[0] * REAL(y)[0] ) ;
' )
package.skeleton( "foo", fx )


functions <- cxxfunction( 
	list( 
		ff = signature(), 
		gg = signature( x = "integer", y = "numeric" )
	), 
	c( "return R_NilValue ;", "return ScalarReal( INTEGER(x)[0] * REAL(y)[0] ) ;")
)
package.skeleton( "foobar", functions )

}
}
\keyword{methods}

\name{getDynLib-methods}
\docType{methods}
\alias{getDynLib}
\alias{getDynLib-methods}
\alias{getDynLib,CFunc-method}
\alias{getDynLib,character-method}
\alias{getDynLib,CFuncList-method}
\title{Retrieve the dynamic library (or DLL) associated with a package of 
a function generated by cfunction}
\description{
The \code{getDynLib} function retrieves the dynamic library (or DLL) 
associated with a package or with a function generated by 
\code{\link{cfunction}}
}
\section{Methods}{
\describe{

\item{\code{signature(x = "CFunc")}}{Retrieves the dynamic 
library associated with the function generated by \code{\link{cfunction}}. 
The library is dynamically loaded if necessary. }

\item{\code{signature(x = "CFuncList")}}{Retrieves the dynamic 
library associated with a set of functions generated by \code{\link{cfunction}}. 
The library is dynamically loaded if necessary. }

\item{\code{signature(x = "character")}}{
Retrieves the dynamic library of the given name. This typically refers 
to package names, but can be any name of the list returned by 
\code{\link{getLoadedDLLs}}
}
}}
\seealso{\code{\link{getLoadedDLLs}}, \code{\link{dyn.load}} }
\examples{
\dontrun{
getDynLib( "base" )

f <- cfunction( signature() , "return R_NilValue ;" )
getDynLib( f )

}}
\keyword{methods}

\name{utilities}

\alias{utilities}
\alias{moveDLL}
\alias{moveDLL-methods}
\alias{moveDLL,CFunc-method}
\alias{writeCFunc}
\alias{readCFunc}
\alias{print,CFunc-method}
\alias{print,CFuncList-method}
\alias{code}
\alias{code-methods}
\alias{code,character-method}
\alias{code,CFunc-method}
\alias{code,CFuncList-method}

\title{ Printing, reading and writing compiled function objects }

\description{
  \code{moveDLL} moves the DLL used by a compiled function to a user defined
  location.

  \code{writeCFunc} saves a \code{CFunc} object after the DLL has been moved to
  the desired location using \code{moveDLL}.

  \code{readCFunc} reads a \code{CFunc} object that has been saved using
  \code{writeCFunc}.

  The \code{print} and \code{code} methods respectively print the entire
  object or only the code parts.
}

\usage{
moveDLL(x, ...)
\S4method{moveDLL}{CFunc}(x, name, directory, unload = FALSE, overwrite = FALSE, verbose = FALSE)

writeCFunc(x, file)
readCFunc(file)

\S4method{print}{CFunc}(x)
\S4method{print}{CFuncList}(x)

\S4method{code}{CFunc}(x, linenumbers = TRUE)
\S4method{code}{CFuncList}(x, linenumbers = TRUE)
}

\arguments{

  \item{x}{A \code{CFunc} or \code{CFuncList} object as created by \code{\link{cfunction}}}

  \item{name}{The base of the file name that the DLL should be moved to. The file name
    extension will depend on the operating system used}

  \item{directory}{The directory that the DLL should be written to}

  \item{unload}{In case the new path constructed from \code{name} and
    \code{directory} points to a loaded DLL, should we unload it?}

  \item{overwrite}{In case there is a file at the new path constructed from
    \code{name} and \code{directory} should we overwrite that file?}

  \item{verbose}{Should we print a message stating where the DLL was copied
    if the operation was successful?}

  \item{file}{The file path for writing and reading the object generated by
    \code{\link{cfunction}}. Consider using a file name extension like
    \code{.rda} or \code{.RData} to indicate that this is a serialized
    R object.
  }

  \item{linenumbers}{If \code{TRUE} all code lines will be numbered.}

  \item{\dots}{May be used in future methods}

}

\value{

  Function \code{readDynLib} returns a \code{CFunc} object.

  Function \code{writeDynLib} returns the name of the \code{.CFunc} file that
  was created.
}

\details{

  If you move the DLL to a user defined location with \code{moveDLL}, this will
  keep an on-disk copy of the DLL which will prevent it from being lost at
  session termination - unless written to the session \code{\link{tempdir}}.
  Saving and reloading the \code{CFunc} object with standard tools like
  \code{\link{save}} or \code{\link{saveRDS}} will still loose the pointer to
  the DLL. However, when the DLL has been moved using \code{moveDLL},
  \code{CFunc} objects can be saved by \code{writeCFunc} and restored by
  \code{readCFunc}.

}

\note{

  \itemize{

    \item The code of a \code{CFunc} or \code{CFuncList} object \code{x} can be extracted
      (rather than printed), using:

    \code{x@code}.

    \item To write the code to a file (here called \code{"fn"}), without the
      new-line character \code{"\n"}:

    \code{write (strsplit(x, "\n")[[1]], file = "fn")}

  }

}

\seealso{

  \code{\link{getDynLib}}

}

\examples{

x <- as.numeric(1:10)
n <- as.integer(10)

code <- "
      integer i
      do 1 i=1, n(1)
    1 x(i) = x(i)**3
"
cubefn <- cfunction(signature(n="integer", x="numeric"), code,
  convention=".Fortran")
code(cubefn)

cubefn(n, x)$x

\dontrun{
# The following code is exempted from the automated tests of example code, as
# it writes to the users home directory.
# The following writes the DLL, e.g. cubefn.so on Linux/Unix or cubefn.dll
# on Windows
moveDLL(cubefn, name = "cubefn", directory = "~")
path <- file.path("~", "cubefn.rda")
writeCFunc(cubefn, path)
rm(cubefn)

# Now you can start a fresh R session and load the function
library(inline)
path <- file.path("~", "cubefn.rda")
cfn <- readCFunc(path)
cfn(3, 1:3)$x
}
}

\author{
  Karline Soetaert and Johannes Ranke
}

\keyword{file}

=pod

=for comment
DO NOT EDIT. This Pod was generated by Swim v0.1.38.
See http://github.com/ingydotnet/swim-pm#readme

=encoding utf8

=head1 NAME

Inline-API - How to bind a programming language to Perl using Inline.pm

=head1 SYNOPSIS

    #!/usr/bin/perl

    use Inline Foo;
    say_it('foo');  # Use Foo to print "Hello, Foo"

    __Foo__
    foo-sub say_it {
        foo-my $foo = foo-shift;
        foo-print "Hello, $foo\n";
    }

=head1 DESCRIPTION

So you think Inline C is pretty cool, but what you really need is for Perl to
work with the brand new programming language "Foo". Well you're in luck.
C<Inline.pm> has support for adding your own Inline Language Support Module
(B<ILSM>), like C<Inline::Foo>.

Inline has always been intended to work with lots of different programming
languages. Many of the details can be shared between implementations, so that
C<Inline::Java> has a similar interface to C<Inline::ASM>. All of the common
code is in C<Inline.pm>.

Language specific modules like C<Inline::Python> are subclasses of
C<Inline.pm>. They can inherit as much of the common behaviour as they want,
and provide specific behaviour of their own. This usually comes in the form of
Configuration Options and language specific compilation.

The Inline C support is probably the best boilerplate to copy from. Since
version 0.30 all C support was isolated into the module C<Inline::C> and the
parsing grammar is further broken out into C<Inline::C::grammar>. All of these
components come with the Inline distribution.

This POD gives you all the details you need for implementing an ILSM. For
further assistance, contact inline@perl.org See ["SEE ALSO"] below.

We'll examine the joke language Inline::Foo which is distributed with Inline.
It actually is a full functioning ILSM. I use it in Inline's test harness to
test base Inline functionality. It is very short, and can help you get your
head wrapped around the Inline API.

=head1 A SKELETON

For the remainder of this tutorial, let's assume we're writing an ILSM for the
ficticious language C<Foo>. We'll call it C<Inline::Foo>. Here is the entire
(working) implementation.

    package Inline::Foo;
    use strict;
    $Inline::Foo::VERSION = '0.01';
    @Inline::Foo::ISA = qw(Inline);
    require Inline;
    use Carp;

    #===========================================================
    # Register Foo as an Inline Language Support Module (ILSM)
    #===========================================================
    sub register {
        return {
            language => 'Foo',
            aliases => ['foo'],
            type => 'interpreted',
            suffix => 'foo',
           };
    }

    #===========================================================
    # Error messages
    #===========================================================
    sub usage_config {
        my ($key) = @_;
        "'$key' is not a valid config option for Inline::Foo\n";
    }

    sub usage_config_bar {
        "Invalid value for Inline::Foo config option BAR";
    }

    #===========================================================
    # Validate the Foo Config Options
    #===========================================================
    sub validate {
        my $o = shift;
        $o->{ILSM}{PATTERN} ||= 'foo-';
        $o->{ILSM}{BAR} ||= 0;
        while (@_) {
        my ($key, $value) = splice @_, 0, 2;
        if ($key eq 'PATTERN') {
            $o->{ILSM}{PATTERN} = $value;
            next;
        }
        if ($key eq 'BAR') {
            croak usage_config_bar
              unless $value =~ /^[01]$/;
            $o->{ILSM}{BAR} = $value;
            next;
        }
        croak usage_config($key);
        }
    }

    #===========================================================
    # Parse and compile Foo code
    #===========================================================
    sub build {
        my $o = shift;
        my $code = $o->{API}{code};
        my $pattern = $o->{ILSM}{PATTERN};
        $code =~ s/$pattern//g;
        $code =~ s/bar-//g if $o->{ILSM}{BAR};
        sleep 1;             # imitate compile delay
        {
            package Foo::Tester;
            eval $code;
        }
        croak "Foo build failed:\n$@" if $@;
        my $path = "$o->{API}{install_lib}/auto/$o->{API}{modpname}";
        my $obj = $o->{API}{location};
        $o->mkpath($path) unless -d $path;
        open FOO_OBJ, "> $obj"
          or croak "Can't open $obj for output\n$!";
        print FOO_OBJ $code;
        close \*FOO_OBJ;
    }

    #===========================================================
    # Only needed for interpreted languages
    #===========================================================
    sub load {
        my $o = shift;
        my $obj = $o->{API}{location};
        open FOO_OBJ, "< $obj"
          or croak "Can't open $obj for output\n$!";
        my $code = join '', <FOO_OBJ>;
        close \*FOO_OBJ;
        eval "package $o->{API}{pkg};\n$code";
        croak "Unable to load Foo module $obj:\n$@" if $@;
    }

    #===========================================================
    # Return a small report about the Foo code.
    #===========================================================
    sub info {
        my $o = shift;
        my $text = <<'END';
    This is a small report about the Foo code. Perhaps it contains
    information about the functions the parser found which will be
    bound to Perl. It will get included in the text produced by the
    Inline 'INFO' command.
    END
        return $text;
    }

    1;

Except for C<load()>, the subroutines in this code are mandatory for an ILSM.
What they do is described below. A few things to note:

=over

=item * C<Inline::Foo> must be a subclass of Inline. This is accomplished with:

    @Inline::Foo::ISA = qw(Inline);

=item * The line 'C<require Inline;>' is not necessary. But it is there to remind you not to say 'C<use Inline;>'. This will not work.

=item * Remember, it is not valid for a user to say:

    use Inline::Foo;

C<Inline.pm> will detect such usage for you in its C<import> method, which is
automatically inherited since C<Inline::Foo> is a subclass.

=item * In the build function, you normally need to parse your source code. Inline::C uses Parse::RecDescent to do this. Inline::Foo simply uses eval. (After we strip out all occurrences of 'foo-').

An alternative parsing method that works well for many ILSMs (like Java and
Python) is to use the language's compiler itself to parse for you. This works
as long as the compiler can be made to give back parse information.

=back

=head1 THE INLINE API

This section is a more formal specification of what functionality you'll need
to provide to implement an ILSM.

When Inline determines that some C<Foo> code needs to be compiled it will
automatically load your ILSM module. It will then call various subroutines
which you need to supply. We'll call these subroutines "callbacks".

You will need to provide the following 5 callback subroutines.

=head2 The register() Callback

This subroutine receives no arguments. It returns a reference to a hash of
ILSM meta-data. Inline calls this routine only when it is trying to detect new
ILSM-s that have been installed on a given system. Here is an example of the
has ref you would return for Foo:

    {
        language => 'Foo',
        aliases => ['foo'],
        type => 'interpreted',
        suffix => 'foo',
    };

The meta-data items have the following meanings:

=over

=item language

This is the proper name of the language. It is usually implemented as
C<Inline::X> for a given language 'X'.

=item aliases

This is a reference to an array of language name aliases. The proper name of a
language can only contain word characters. L<A-Za-z0-9_> An alias can contain
any characters except whitespace and quotes. This is useful for names like
'C++' and 'C#'.

=item type

Must be set to 'compiled' or 'interpreted'. Indicates the category of
the language.

=item suffix

This is the file extension for the cached object that will be created. For
'compiled' languages, it will probably be 'so' or 'dll'. The appropriate value
is in C<Config.pm>.

For interpreted languages, this value can be whatever you want. Python uses
C<pydat>. Foo uses C<foo>.

=back

=head2 The validate() Callback

This routine gets passed all configuration options that were not already
handled by the base Inline module. The options are passed as key/value pairs.
It is up to you to validate each option and store its value in the Inline
object (which is also passed in). If a particular option is invalid, you
should croak with an appropriate error message.

Note that all the keywords this routine receives will be converted to upper-
case by C<Inline>, whatever case the program gave.

=head2 The build() Callback

This subroutine is responsible for doing the parsing and compilation of the
Foo source code. The Inline object is passed as the only argument. All
pertinent information will be stored in this object. C<build()> is required to
create a cache object of a specific name, or to croak with an appropriate
error message.

This is the meat of your ILSM. Since it will most likely be quite complicated,
it is probably best that you study an existing ILSM like C<Inline::C>.

=head2 The load() Callback

This method only needs to be provided for interpreted languages. It's
responsibility is to start the interpreter.

For compiled languages, the load routine from C<Inline.pm> is called which
uses C<DynaLoader> to load the shared object or DLL.

=head2 The info() Callback

This method is called when the user makes use of the C<INFO> shortcut. You
should return a string containing a small report about the Inlined code.

=head1 THE INLINE OBJECT

C<Inline.pm> creates a hash based Perl object for each section of Inlined
source code it receives. This object contains lots of information about the
code, the environment, and the configuration options used.

This object is a hash that is broken into several subhashes. The only two
subhashes that an ILSM should use at all are $o->{API} and $o->{ILSM}. The
first one contains all of the information that Inline has gather for you
in order for you to create/load a cached object of your design. The second
one is a repository where your ILSM can freely store data that it might
need later on.

This section will describe all of the Inline object "API" attributes.

=over

=item The code Attribute

This the actual source code passed in by the user. It is stored as one
long string.

=item The language Attribute

The proper name of the language being used.

=item The language_id Attribute

The language name specified by the user. Could be 'C++' instead of 'CPP'.

=item The module Attribute

This is the shared object's file name.

=item The modfname Attribute

This is the shared object's file name.

=item The modpname Attribute

This is the shared object's installation path extension.

=item The version Attribute

The version of C<Inline.pm> being used.

=item The pkg Attribute

The Perl package from which this invocation pf Inline was called.

=item The install_lib Attribute

This is the directory to write the shared object into.

=item The build_dir Attribute

This is the directory under which you should write all of your build
related files.

=item The script Attribute

This is the name of the script that invoked Inline.

=item The location Attribute

This is the full path name of the executable object in question.

=item The suffix Attribute

This is the shared library extension name. (Usually 'so' or 'dll').

=back

=head1 THE INLINE NAMESPACE

C<Inline.pm> has been set up so that anyone can write their own language
support modules. It further allows anyone to write a different implementation
of an existing Inline language, like C for instance. You can distribute that
module on the CPAN.

If you have plans to implement and distribute an Inline module, I would ask
that you please work with the Inline community. We can be reached at the
Inline mailing list: inline@perl.org (Send mail to inline-subscribe@perl.org
to subscribe). Here you should find the advice and assistance needed to make
your module a success.

The Inline community will decide if your implementation of COBOL will be
distributed as the official C<Inline::COBOL> or should use an alternate
namespace. In matters of dispute, I (Ingy döt Net) retain final authority.
(and I hope not to need use of it :-) Actually modules@perl.org retains the
B<final> authority.

But even if you want to work alone, you are free and welcome to write and
distribute Inline language support modules on CPAN. You'll just need to
distribute them under a different package name.

=head1 SEE ALSO

For generic information about Inline, see L<Inline>.

For information about using Inline with C see L<Inline::C>.

For information on supported languages and platforms see L<Inline-Support>.

Inline's mailing list is inline@perl.org

To subscribe, send email to inline-subscribe@perl.org

=head1 AUTHOR

Ingy döt Net <ingy@cpan.org>

=head1 COPYRIGHT

Copyright 2000-2015. Ingy döt Net.

Copyright 2008, 2010, 2011. Sisyphus.

This program is free software; you can redistribute it and/or modify it under
the same terms as Perl itself.

See L<http://www.perl.com/perl/misc/Artistic.html>

=cut

\name{plugins}
\alias{getPlugin}
\alias{registerPlugin}
\title{
Plugin system for cxxfunction
}
\description{
\code{\link{cxxfunction}} uses a plugin system to assembly the 
code that it compiles. These functions allow to register and get 
plugins by their name. 
}
\usage{
getPlugin(name, ...)
registerPlugin(name, plugin)
}
\arguments{
  \item{name}{name of the plugin. }
  \item{\dots}{Further argments to pass to the plugin. }
  \item{plugin}{plugin function. }
}
\details{
plugins are functions that return a list with : 
\describe{
	\item{includes}{mandatory. it is included at the top of the compiled file by \code{\link{cxxfunction}} }
	\item{body}{optional. a function that takes one argument (the body of the c++ function) and returned
	a modified version of the body. The "Rcpp" plugin uses this to surround the code with the 
	\code{BEGIN_RCPP} and \code{END_RCPP} macros
	}	
	\item{LinkingTo}{optional. character vector containing the list of packages
	that the code needs to link to. This adds the include path of the given 
	packages. The "Rcpp" and "RcppArmadillo" plugins use this.
	}
	\item{env}{optional. named list of environment variables. For example, the "Rcpp" 
	plugin uses this to add Rcpp user library to the \code{PKG_LIBS} environment
	variable.
	}
}

plugins can be manually registered using the \code{registerPlugin}
function. Alternatively, a package may supply an inline plugin 
implicitely by defining a function called \code{inlineCxxPlugin}, which 
does not necessarily need to be exported from the namespace of the package.

Known packages implementing this scheme include \code{Rcpp} and 
\code{RcppArmadillo}. 

}
\value{
	\code{getPlugin} retrieves the plugin and invokes it with the
	\dots arguments
	
	\code{registerPlugin} does not return anything.
}
\seealso{
	\code{\link{cxxfunction}}
}
\examples{
\dontrun{
getPlugin( "Rcpp" )
}
}
\keyword{programming}
\keyword{interface}

=head1 NAME 

Inline-FAQ - The Inline FAQ

=head1 DESCRIPTION

Welcome to the official Inline FAQ. In this case, B<FAQ> means:

    Formerly Answered Questions

This is a collection of old, long-winded emails that myself and others
have sent to the Inline mailing list. (inline@perl.org) They have been
reviewed and edited for general Inline edification. Some of them may be
related to a specific language. They are presented here in a traditional
FAQ layout.

=head1 General Inline

Since there is only a handful of content so far, all FAQs are currently
under this heading.

=head2 How disposable is a .Inline or _Inline directory?

I probably need to be more emphatic about the roll of _Inline/ cache
directories. Since they are created automatically, they are completely
disposable. I delete them all the time. And it is fine to have a
different one for each project. In fact as long as you don't have
~/.Inline/ defined, Inline will create a new ./_Inline directory. You
can move that to ./.Inline and it will continue to work if you want to
give it more longevity and hide it from view. There is a long
complicated list of rules about how [_.]Inline/ directories are
used/created. But it was designed to give you the most
flexibility/ease-of-use. Never be afraid to nuke 'em. They'll just pop
right back next time. :)

=head2 Whatever happened to the SITE_INSTALL option?

SITE_INSTALL is gone. I was going to leave it in and change the
semantics, but thought it better to remove it, so people wouldn't try to
use it the old way. There is now _INSTALL_ (but you're not supposed to
know that :). It works magically through the use of Inline::MakeMaker. I
explained this earlier but it's worth going through again because it's
the biggest change for 0.40. Here's how to 'permanently' install an
Inline extension (Inline based module) with 0.40:

    1) Create a module with Inline.
    2) Test it using the normal/local _Inline/ cache.
    3) Create a Makefile.PL (like the one produced by h2xs)
    4) Change 'use ExtUtils::MakeMaker' to 'use Inline::MakeMaker'
    5) Change your 'use Inline C => DATA' to 'use Inline C => DATA => NAME
       => Foo => VERSION => 1.23'
    6) Make sure NAME matches your package name ('Foo'), or begins with
       'Foo::'.
    7) Make sure VERSION matches $Foo::VERSION. This must be a string (not a
       number) matching /^\d\.\d\d$/
    8) Do the perl/make/test/install dance (thanks binkley :)

With Inline 0.41 (or thereabouts) you can skip steps 3 & 4, and just
say 'perl -MInline=INSTALL ./Foo.pm'. This will work for non-Inline
modules too. It will become the defacto standard (since there is no easy
standard) way of installing a Perl module. It will allow Makefile.PL
parameters 'perl -MInline=INSTALL ./Foo.pm - PREFIX=/home/ingy/perl' and
things like that. It will also make use of a MANIFEST if you provide
one.

=head2 How do I create a binary distribution using Inline?

I've figured out how to create and install a PPM binary distribution;
with or without distributing the C code! And I've decided to share it
with all of you :)

NOTE: Future versions of Inline will make this process a one line
command. But for now just use this simple recipe.

---

The Inline 0.40 distribution comes with a sample extension module called
Math::Simple. Theoretically you could distribute this module on CPAN. It
has all the necessary support for installation. You can find it in
Inline-0.40/modules/Math/Simple/. Here are the steps for converting this
into a binary distribution *without* C source code.

NOTE: The recipient of this binary distribution will need to have the
PPM.pm module installed. This module requires a lot of other CPAN
modules. ActivePerl (available for Win32, Linux, and Solaris) has all of
these bundled. While ActivePerl isn't required, it makes things (a
lot) easier.

1) cd Inline-0.40/Math/Simple/

2) Divide Simple.pm into two files:

    ---8<--- (Simple.pm)
    package Math::Simple;
    use strict;
    require Exporter;
    @Math::Simple::ISA = qw(Exporter);
    @Math::Simple::EXPORT = qw(add subtract);
    $Math::Simple::VERSION = '1.23';
    
    use Inline (C => 'src/Simple.c' =>
                NAME => 'Math::Simple',
                VERSION => '1.23',
               );
    1;
    ---8<---
    ---8<--- (src/Simple.c)
    int add (int x, int y) {
        return x + y;
    }
    
    int subtract (int x, int y) {
        return x - y;
    }
    ---8<---

So now you have the Perl in one file and the C in the other. The C code
must be in a subdirectory.

3) 
Note that I also changed the term 'DATA' to the name of the C file. This
will work just as if the C were still inline.

4) Run 'perl Makefile.PL'

5) Run 'make test'

6) Get the MD5 key from 'blib/arch/auto/Math/Simple/Simple.inl'

7) 
Edit 'blib/lib/Math/Simple.pm'. Change 'src/Simple.c' to
'02c61710cab5b659efc343a9a830aa73' (the MD5 key)

8) Run 'make ppd'

9) 
Edit 'Math-Simple.ppd'. Fill in AUTHOR and ABSTRACT if you wish. Then
change:

    <CODEBASE HREF="" />

to

    <CODEBASE HREF="Math-Simple.tar.gz" />

10) Run:

    tar cvf Math-Simple.tar blib
    gzip --best Math-Simple.tar

11) 
Run:

    tar cvf Math-Simple-1.23.tar Math-Simple.ppd Math-Simple.tar.gz
    gzip --best Math-Simple-1.23.tar

12) Distribute Math-Simple-1.23.tar.gz with the following instructions:

A) Run: 

    gzip -d Math-Simple-1.23.tar.gz
    tar xvzf Math-Simple-1.23.tar

B) Run 'ppm install Math-Simple.ppd'

C) Delete Math-Simple.tar and Math-Simple.ppd.

D) Test with:

    perl -MMath::Simple -le 'print add(37, 42)'

---

That's it. The process should also work with zip instead of tar, but I
haven't tried it.

The recipient of the binary must have Perl built with a matching
architecture. Luckily, ppm will catch this.

For a binary dist *with* C source code, simply omit steps 2, 3, 6, and
7.

If this seems too hard, then in a future version you should be able to
just type:

    make ppm

=cut
